@@ -35,18 +35,20 @@ import locale
 import signal
 import os
 import re
+import subprocess
 
 from gi.repository import GLib
 
 from . import brltablenames
 from . import cmdnames
 from . import debug
-from . import eventsynthesizer
 from . import logger
 from . import orca_state
 from . import settings
 from . import settings_manager
 
+from .ax_event_synthesizer import AXEventSynthesizer
+from .ax_object import AXObject
 from .orca_platform import tablesdir
 
 _logger = logger.getLogger()
@@ -56,47 +58,49 @@ _settingsManager = settings_manager.getManager()
 
 try:
     msg = "BRAILLE: About to import brlapi."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     import brlapi
     _brlAPI = None
     _brlAPIAvailable = True
+    _brlAPIConnectable = True
     _brlAPIRunning = False
     _brlAPISourceId = 0
-except:
+except Exception:
     msg = "BRAILLE: Could not import brlapi."
-    debug.println(debug.LEVEL_WARNING, msg, True)
+    debug.printMessage(debug.LEVEL_WARNING, msg, True)
     _brlAPIAvailable = False
     _brlAPIRunning = False
+    _brlAPIConnectable = False
 else:
-    msg = "BRAILLE: brlapi imported %s" % brlapi
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: brlapi imported", brlapi]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
 try:
     msg = "BRAILLE: About to import louis."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     import louis
-except:
+except Exception:
     msg = "BRAILLE: Could not import liblouis"
-    debug.println(debug.LEVEL_WARNING, msg, True)
+    debug.printMessage(debug.LEVEL_WARNING, msg, True)
     louis = None
 else:
-    msg = "BRAILLE: liblouis imported %s" % louis
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: liblouis imported", louis]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-    msg = "BRAILLE: tables location: %s" % tablesdir
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: tables location:", tablesdir]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     # TODO: Can we get the tablesdir info at runtime?
     if not tablesdir:
         msg = "BRAILLE: Disabling liblouis due to unknown table location." \
               "This usually means orca was built before liblouis was installed."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         louis = None
 
 try:
     from . import brlmon
-except:
+except Exception:
     settings.enableBrailleMonitor = False
 
 
@@ -249,17 +253,17 @@ def listTables():
 
 def getDefaultTable():
     userLocale = locale.getlocale(locale.LC_MESSAGES)[0]
-    msg = "BRAILLE: User locale is %s" % userLocale
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: User locale is", userLocale]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if userLocale in (None, "C"):
         userLocale = locale.getdefaultlocale()[0]
-        msg = "BRAILLE: Default locale is %s" % userLocale
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["BRAILLE: Default locale is", userLocale]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if userLocale in (None, "C"):
         msg = "BRAILLE: Locale cannot be determined. Falling back on 'en-us'"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         language = "en-us"
     else:
         language = "-".join(userLocale.split("_")).lower()
@@ -267,8 +271,8 @@ def getDefaultTable():
     try:
         tables = [x for x in os.listdir(tablesdir) if x[-4:] in (".utb", ".ctb")]
     except OSError:
-        msg = "BRAILLE: Exception calling os.listdir for %s" % tablesdir
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["BRAILLE: Exception calling os.listdir for", tablesdir]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return ""
 
     # Some of the tables are probably not a good choice for default table....
@@ -281,10 +285,12 @@ def getDefaultTable():
     # for the largest group of users; not the perfect default for all users.
     prefer = ["g1", "g2", "comp6", "comp8"]
 
-    isCandidate = lambda t: t.startswith(language) and not any(e in t for e in exclude)
+    def isCandidate(t):
+        return t.startswith(language) and not any(e in t for e in exclude)
+
     tables = list(filter(isCandidate, tables))
-    msg = "BRAILLE: %i candidate tables for locale found: %s" % (len(tables), ", ".join(tables))
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE:", len(tables), "candidate tables for locale found:", ', '.join(tables)]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if not tables:
         return ""
@@ -299,8 +305,8 @@ def getDefaultTable():
 
 if louis:
     _defaultContractionTable = getDefaultTable()
-    msg = "BRAILLE: Default contraction table is: %s" % _defaultContractionTable
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: Default contraction table is:", _defaultContractionTable]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
 def _printBrailleEvent(level, command):
     """Prints out a Braille event.  The given level may be overridden
@@ -313,7 +319,7 @@ def _printBrailleEvent(level, command):
 
     debug.printInputEvent(
         level,
-        "BRAILLE EVENT: %s" % repr(command))
+        f"BRAILLE EVENT: {repr(command)}")
 
 class Region:
     """A Braille region to be displayed on the display.  The width of
@@ -344,8 +350,8 @@ class Region:
         if self.contracted:
             self.contractionTable = settings.brailleContractionTable or _defaultContractionTable
             if string.strip():
-                msg = "BRAILLE: Contracting '%s' with table %s" % (string, self.contractionTable)
-                debug.println(debug.LEVEL_INFO, msg, True)
+                tokens = ["BRAILLE: Contracting '", string, "' with table", self.contractionTable]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
             self.string, self.inPos, self.outPos, self.cursorOffset = \
                          self.contractLine(self.rawLine,
@@ -353,12 +359,17 @@ class Region:
         else:
             if string.strip():
                 if not settings.enableContractedBraille:
-                    msg = "BRAILLE: Not contracting '%s' because contracted braille is not enabled." % string
-                    debug.println(debug.LEVEL_INFO, msg, True)
+                    msg = (
+                        f"BRAILLE: Not contracting '{string}' "
+                        f"because contracted braille is not enabled."
+                    )
+                    debug.printMessage(debug.LEVEL_INFO, msg, True)
                 else:
-                    msg = "BRAILLE: Not contracting '%s' due to problem with liblouis." % string
-                    debug.println(debug.LEVEL_WARNING, msg, True)
-
+                    msg = (
+                        f"BRAILLE: Not contracting '{string}' "
+                        f"due to problem with liblouis."
+                    )
+                    debug.printMessage(debug.LEVEL_INFO, msg, True)
             self.string = self.rawLine
             self.cursorOffset = cursorOffset
 
@@ -520,26 +531,24 @@ class Component(Region):
            grabFocusBeforeRouting(self.accessible, offset):
             try:
                 self.accessible.queryComponent().grabFocus()
-            except:
+            except Exception:
                 pass
 
+        if AXObject.do_action(self.accessible, 0):
+            return
+
+        # Do a mouse button 1 click if we have to.  For example, page tabs
+        # don't have any actions but we want to be able to select them with
+        # the cursor routing key.
         try:
-            action = self.accessible.queryAction()
-        except:
-            # Do a mouse button 1 click if we have to.  For example, page tabs
-            # don't have any actions but we want to be able to select them with
-            # the cursor routing key.
-            #
-            debug.println(debug.LEVEL_FINEST,
-                          "braille.Component.processRoutingKey: no action")
-            try:
-                eventsynthesizer.clickObject(self.accessible, 1)
-            except:
-                debug.println(debug.LEVEL_SEVERE,
-                              "Could not process routing key:")
-                debug.printException(debug.LEVEL_SEVERE)
+            result = AXEventSynthesizer.click_object(self.accessible, 1)
+        except Exception as error:
+            tokens = ["ERROR: Could not process routing key:", error]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
         else:
-            action.doAction(0)
+            if not result:
+                msg = "INFO: Processing routing key failed"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
 
 class Link(Component):
     """A subclass of Component backed by an accessible.  This Region will be
@@ -712,7 +721,7 @@ class Text(Region):
         """
 
         try:
-            text = self.accessible.queryText()
+            self.accessible.queryText()
         except NotImplementedError:
             return ''
 
@@ -726,12 +735,16 @@ class Text(Region):
         selIndicator = settings.brailleSelectorIndicator
         linkIndicator = settings.brailleLinkIndicator
         script = orca_state.activeScript
+        if script is None:
+            msg = "BRAILLE: Cannot get attribute mask without active script."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            return ""
 
         if getLinkMask and linkIndicator != settings.BRAILLE_UNDERLINE_NONE:
             try:
                 hyperText = self.accessible.queryHypertext()
                 nLinks = hyperText.getNLinks()
-            except:
+            except Exception:
                 nLinks = 0
 
             n = 0
@@ -741,7 +754,7 @@ class Text(Region):
                     for i in range(link.startIndex, link.endIndex):
                         try:
                             regionMask[i] |= linkIndicator
-                        except:
+                        except Exception:
                             pass
                 n += 1
 
@@ -943,7 +956,8 @@ class Line:
                     displayWidths = wordLength // _displaySize[0]
                     if displayWidths:
                         for i in range(displayWidths):
-                            ranges.append([start + i * _displaySize[0], start + (i+1) * _displaySize[0]])
+                            ranges.append([start + i * _displaySize[0],
+                                            start + (i+1) * _displaySize[0]])
                         if wordLength % _displaySize[0]:
                             span = [start + displayWidths * _displaySize[0], end]
                         else:
@@ -1139,16 +1153,16 @@ def _idleBraille():
     if not idle:
         try:
             msg = "BRAILLE: Attempting to idle braille."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, 0)
             idle = True
-        except:
+        except Exception:
             msg = "BRAILLE: Idling braille failled. This requires BrlAPI >= 0.8."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             pass
         else:
             msg = "BRAILLE: Idling braille succeeded."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     return idle
 
@@ -1164,39 +1178,39 @@ def _clearBraille():
         try:
             _brlAPI.writeText("", 0)
             _idleBraille()
-        except:
+        except Exception:
             msg = "BRAILLE: BrlTTY seems to have disappeared."
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
             shutdown()
 
 def _enableBraille():
     """Re-enable Braille output after making it idle or clearing it"""
     global idle
 
-    msg = "BRAILLE: Enabling braille. BrlAPI running: %s" % _brlAPIRunning
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: Enabling braille. BrlAPI running:", _brlAPIRunning]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if not _brlAPIRunning:
         msg = "BRAILLE: Need to initialize first."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         init(_callback)
 
     if _brlAPIRunning:
         if idle:
             msg = "BRAILLE: Is running, but idling."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             try:
                 # Restore default priority
                 msg = "BRAILLE: Attempting to de-idle braille."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 _brlAPI.setParameter(brlapi.PARAM_CLIENT_PRIORITY, 0, False, 50)
                 idle = False
-            except:
+            except Exception:
                 msg = "BRAILLE: could not restore priority"
-                debug.println(debug.LEVEL_WARNING, msg, True)
+                debug.printMessage(debug.LEVEL_WARNING, msg, True)
             else:
                 msg = "BRAILLE: De-idle succeeded."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
 
 def disableBraille():
     """Hand off control to other screen readers, shutting down the BrlAPI
@@ -1204,24 +1218,24 @@ def disableBraille():
 
     global idle
 
-    msg = "BRAILLE: Disabling braille. BrlAPI running: %s" % _brlAPIRunning
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: Disabling braille. BrlAPI running:", _brlAPIRunning]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if _brlAPIRunning and not idle:
         msg = "BRAILLE: BrlApi running and not idle."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         if not _idleBraille() and not _settingsManager.getSetting('enableBraille'):
             # BrlAPI before 0.8 and we really want to shut down
             msg = "BRAILLE: could not go idle, completely shut down"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             shutdown()
 
 def checkBrailleSetting():
     """Disable Braille if it got disabled in the preferences"""
 
     msg = "BRAILLE: Checking braille setting."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     if not _settingsManager.getSetting('enableBraille'):
         disableBraille()
@@ -1256,8 +1270,8 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
     global _monitor
     global _lastTextInfo
 
-    msg = "BRAILLE: Refresh. Pan: %s target: %i" % (panToCursor, targetCursorCell)
-    debug.println(debug.LEVEL_INFO, msg, True)
+    msg = f"BRAILLE: Refresh. Pan: {panToCursor} target: {targetCursorCell}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     if stopFlash:
         killFlash(restoreSaved=False)
@@ -1267,7 +1281,7 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
        and not _settingsManager.getSetting('enableBrailleMonitor'):
         if _brlAPIRunning:
             msg = "BRAILLE: FIXME - Braille disabled, but not properly shut down."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             shutdown()
         _lastTextInfo = (None, 0, 0, 0)
         return
@@ -1300,8 +1314,8 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
 
     if targetCursorCell < 0:
         targetCursorCell = _displaySize[0] + targetCursorCell + 1
-        msg = "BRAILLE: Adjusted targetCursorCell to: %i" % targetCursorCell
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f"BRAILLE: Adjusted targetCursorCell to: {targetCursorCell}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     # If there is no target cursor cell and panning to cursor was
     # requested, then try to set one.  We
@@ -1313,44 +1327,44 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
     if panToCursor and targetCursorCell == 0 and onSameLine:
         if lastCursorCell == 0:
             msg = "BRAILLE: Not adjusting targetCursorCell. User panned caret out of view."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         elif lastCaretOffset == currentCaretOffset:
             targetCursorCell = lastCursorCell
             msg = "BRAILLE: Setting targetCursorCell to previous value. Caret hasn't moved."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         elif lastCaretOffset < currentCaretOffset:
             newLocation = lastCursorCell + (currentCaretOffset - lastCaretOffset)
             if newLocation <= _displaySize[0]:
-                msg = "BRAILLE: Setting targetCursorCell based on offset: %i" % newLocation
-                debug.println(debug.LEVEL_INFO, msg, True)
+                msg = f"BRAILLE: Setting targetCursorCell based on offset: {newLocation}"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 targetCursorCell = newLocation
             else:
                 msg = "BRAILLE: Setting targetCursorCell to end of display."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 targetCursorCell = _displaySize[0]
         elif lastCaretOffset > currentCaretOffset:
             newLocation = lastCursorCell - (lastCaretOffset - currentCaretOffset)
             if newLocation >= 1:
-                msg = "BRAILLE: Setting targetCursorCell based on offset: %i" % newLocation
-                debug.println(debug.LEVEL_INFO, msg, True)
+                msg = f"BRAILLE: Setting targetCursorCell based on offset: {newLocation}"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 targetCursorCell = newLocation
             else:
                 msg = "BRAILLE: Setting targetCursorCell to start of display."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 targetCursorCell = 1
 
     # Now, we figure out the 0-based offset for where the cursor actually is in the string.
 
     line = _lines[viewport[1]]
     [string, focusOffset, attributeMask, ranges] = line.getLineInfo(getLinkMask)
-    msg = "BRAILLE: Line %i: '%s' focusOffset: %i %s" % (viewport[1], string, focusOffset, ranges)
-    debug.println(debug.LEVEL_INFO, msg, True)
+    msg = f"BRAILLE: Line {viewport[1]}: '{string}' focusOffset: {focusOffset} {ranges}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     cursorOffset = -1
     if focusOffset >= 0:
         cursorOffset = focusOffset + _regionWithFocus.cursorOffset
-        msg = "BRAILLE: Cursor offset in line string is: %i" % cursorOffset
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f"BRAILLE: Cursor offset in line string is: {cursorOffset}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     # Now, if desired, we'll automatically pan the viewport to show
     # the cursor.  If there's no targetCursorCell, then we favor the
@@ -1359,29 +1373,29 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
     #
     if panToCursor and (cursorOffset >= 0):
         if len(string) <= _displaySize[0] and cursorOffset < _displaySize[0]:
-            msg = "BRAILLE: Not adjusting offset %i. Cursor offset fits on display." % viewport[0]
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f"BRAILLE: Not adjusting offset {viewport[0]}. Cursor offset fits on display."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         elif targetCursorCell:
             viewport[0] = max(0, cursorOffset - targetCursorCell + 1)
-            msg = "BRAILLE: Adjusting offset to %i based on targetCursorCell" % viewport[0]
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f"BRAILLE: Adjusting offset to {viewport[0]} based on targetCursorCell."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         elif cursorOffset < viewport[0]:
             viewport[0] = max(0, cursorOffset)
-            msg = "BRAILLE: Adjusting offset to %i (cursor on left)" % viewport[0]
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f"BRAILLE: Adjusting offset to {viewport[0]} (cursor on left)"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         elif cursorOffset >= (viewport[0] + _displaySize[0]):
             viewport[0] = max(0, cursorOffset - _displaySize[0] + 1)
-            msg = "BRAILLE: Adjusting offset to %i (cursor beyond display end)" % viewport[0]
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f"BRAILLE: Adjusting offset to {viewport[0]} (cursor beyond display end)"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         else:
             rangeForOffset = _getRangeForOffset(cursorOffset)
             viewport[0] = max(0, rangeForOffset[0])
-            msg = "BRAILLE: Adjusting offset to %i (unhandled condition)" % viewport[0]
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f"BRAILLE: Adjusting offset to {viewport[0]} (unhandled condition)"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             if cursorOffset >= (viewport[0] + _displaySize[0]):
                 viewport[0] = max(0, cursorOffset - _displaySize[0] + 1)
-                msg = "BRAILLE: Readjusting offset to %i (cursor beyond display end)" % viewport[0]
-                debug.println(debug.LEVEL_INFO, msg, True)
+                msg = f"BRAILLE: Readjusting offset to {viewport[0]} (cursor beyond display end)"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     startPos, endPos = _adjustForWordWrap(targetCursorCell)
     viewport[0] = startPos
@@ -1395,13 +1409,12 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
     else:
         cursorCell += 1 # Normalize to 1-based offset
 
-    logLine = "BRAILLE LINE:  '%s'" % string
-    debug.println(debug.LEVEL_INFO, logLine, True)
+    logLine = f"BRAILLE LINE:  '{string}'"
+    debug.printMessage(debug.LEVEL_INFO, logLine, True)
     log.info(logLine)
 
-    logLine = "     VISIBLE:  '%s', cursor=%d" % \
-                    (string[startPos:endPos], cursorCell)
-    debug.println(debug.LEVEL_INFO, logLine, True)
+    logLine = f"     VISIBLE:  '{string[startPos:endPos]}', cursor={cursorCell}"
+    debug.printMessage(debug.LEVEL_INFO, logLine, True)
     log.info(logLine)
 
     substring = string[startPos:endPos]
@@ -1447,9 +1460,9 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
 
         try:
             _brlAPI.write(writeStruct)
-        except:
+        except Exception:
             msg = "BRAILLE: BrlTTY seems to have disappeared."
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
             shutdown()
 
     if settings.enableBrailleMonitor:
@@ -1457,8 +1470,8 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True, stopFlash=Tr
             try:
                 _monitor = brlmon.BrlMon(_displaySize[0])
                 _monitor.show_all()
-            except:
-                debug.println(debug.LEVEL_WARNING, "brlmon failed")
+            except Exception:
+                debug.printMessage(debug.LEVEL_WARNING, "brlmon failed")
                 _monitor = None
         if attributeMask:
             subMask = attributeMask[startPos:endPos]
@@ -1599,14 +1612,14 @@ def displayKeyEvent(event):
     lockingStateString = event.getLockingStateString()
     if lockingStateString:
         keyname = event.getKeyName()
-        msg = "%s %s" % (keyname, lockingStateString)
+        msg = f"{keyname} {lockingStateString}"
         displayMessage(msg, flashTime=settings.brailleFlashTime)
 
 def _adjustForWordWrap(targetCursorCell):
     startPos = viewport[0]
     endPos = startPos + _displaySize[0]
-    msg = "BRAILLE: Current range: (%i, %i). Target cell: %i." % (startPos, endPos, targetCursorCell)
-    debug.println(debug.LEVEL_INFO, msg, True)
+    msg = f"BRAILLE: Current range: ({startPos}, {endPos}). Target cell: {targetCursorCell}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     if not _lines or not settings.enableBrailleWordWrap:
         return startPos, endPos
@@ -1615,11 +1628,11 @@ def _adjustForWordWrap(targetCursorCell):
     lineString, focusOffset, attributeMask, ranges = line.getLineInfo()
     ranges = list(filter(lambda x: x[0] <= startPos + targetCursorCell < x[1], ranges))
     if ranges:
-        msg = "BRAILLE: Adjusted range: (%i, %i)" % (ranges[0][0], ranges[-1][1])
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f"BRAILLE: Adjusted range: ({ranges[0][0]}, {ranges[-1][1]})"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         if ranges[-1][1] - ranges[0][0] > _displaySize[0]:
             msg = "BRAILLE: Not adjusting range which is greater than display size"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
         else:
             startPos, endPos = ranges[0][0], ranges[-1][1]
 
@@ -1654,8 +1667,8 @@ def panLeft(panAmount=0):
         panAmount = max(0, min(oldStart - newStart, _displaySize[0]))
 
     viewport[0] = max(0, viewport[0] - panAmount)
-    msg = "BRAILLE: Panning left. Amount: %i (from %i to %i)" % (panAmount, oldX, viewport[0])
-    debug.println(debug.LEVEL_INFO, msg, True)
+    msg = f"BRAILLE: Panning left. Amount: {panAmount} (from {oldX} to {viewport[0]})"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     return oldX != viewport[0]
 
 def panRight(panAmount=0):
@@ -1682,16 +1695,16 @@ def panRight(panAmount=0):
         if newX < len(string):
             viewport[0] = newX
 
-    msg = "BRAILLE: Panning right. Amount: %i (from %i to %i)" % (panAmount, oldX, viewport[0])
-    debug.println(debug.LEVEL_INFO, msg, True)
+    msg = f"BRAILLE: Panning right. Amount: {panAmount} (from {oldX} to {viewport[0]})"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     return oldX != viewport[0]
 
 def panToOffset(offset):
     """Automatically pan left or right to make sure the current offset is
     showing."""
 
-    msg = "BRAILLE: Panning to offset %i. Current offset: %i." % (offset, viewport[0])
-    debug.println(debug.LEVEL_INFO, msg, True)
+    msg = f"BRAILLE: Panning to offset {offset}. Current offset: {viewport[0]}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     while offset < viewport[0]:
         if not panLeft():
@@ -1774,8 +1787,8 @@ def _processBrailleEvent(event):
             # the command was consumed.
             #
             consumed = _callback(event)
-        except:
-            debug.println(debug.LEVEL_WARNING, "Issue processing event:")
+        except Exception:
+            debug.printMessage(debug.LEVEL_WARNING, "Issue processing event:")
             debug.printException(debug.LEVEL_WARNING)
             consumed = False
 
@@ -1790,8 +1803,8 @@ def _brlAPIKeyReader(source, condition):
     """
     try:
         key = _brlAPI.readKey(False)
-    except:
-        debug.println(debug.LEVEL_WARNING, "BrlTTY seems to have disappeared:")
+    except Exception:
+        debug.printMessage(debug.LEVEL_WARNING, "BrlTTY seems to have disappeared:")
         debug.printException(debug.LEVEL_WARNING)
         shutdown()
         return
@@ -1808,34 +1821,84 @@ def setupKeyRanges(keys):
     """
 
     msg = "BRAILLE: Setting up key ranges."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     if not _brlAPIRunning:
         init(_callback)
 
     if not _brlAPIRunning:
         msg = "BRAILLE: Not setting up key ranges: BrlAPI not running."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return
 
     msg = "BRAILLE: Ignoring all key ranges."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     _brlAPI.ignoreKeys(brlapi.rangeType_all, [0])
 
     keySet = [brlapi.KEY_TYPE_CMD | brlapi.KEY_CMD_ROUTE]
 
     msg = "BRAILLE: Enabling commands:"
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     for key in keys:
         keySet.append(brlapi.KEY_TYPE_CMD | key)
 
     msg = "BRAILLE: Sending keys to BrlAPI."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     _brlAPI.acceptKeys(brlapi.rangeType_command, keySet)
 
     msg = "BRAILLE: Key ranges set up."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+def _canConnect():
+    # TODO - JD: We should create a wrapper class for all things brlapi.
+    # In the meantime, don't crash. https://gitlab.gnome.org/GNOME/orca/-/issues/386
+
+    global _brlAPIConnectable
+
+    if not _brlAPIConnectable:
+        return False
+
+    try:
+        import brlapi
+    except ModuleNotFoundError as error:
+        msg = f"BRAILLE: {error}"
+        debug.printMessage(debug.LEVEL_WARNING, msg)
+        _brlAPIConnectable = False
+        return False
+
+    known_errors = [
+        brlapi.ERROR_AUTHENTICATION,
+        brlapi.ERROR_CONNREFUSED,
+        brlapi.ERROR_DEVICEBUSY,
+        brlapi.ERROR_DRIVERERROR,
+        brlapi.ERROR_EMPTYKEY,
+        brlapi.ERROR_EOF,
+        brlapi.ERROR_GAIERR,
+        brlapi.ERROR_ILLEGAL_INSTRUCTION,
+        brlapi.ERROR_INVALID_PACKET,
+        brlapi.ERROR_INVALID_PARAMETER,
+        brlapi.ERROR_LIBCERR,
+        brlapi.ERROR_NOMEM,
+        brlapi.ERROR_OPNOTSUPP,
+        brlapi.ERROR_PROTOCOL_VERSION,
+        brlapi.ERROR_READONLY_PARAMETER,
+        brlapi.ERROR_SUCCESS,
+        brlapi.ERROR_TTYBUSY,
+        brlapi.ERROR_UNKNOWNTTY,
+        brlapi.ERROR_UNKNOWN_INSTRUCTION
+    ]
+
+    cmd = ['python3', '-c', 'import brlapi; brlapi.Connection()']
+    process_result = subprocess.run(cmd, capture_output=True, text=True)
+    if process_result.returncode not in known_errors:
+        context_msg = "BRAILLE: Error while trying to establish BrlAPI connection:"
+        error_msg = process_result.stderr.strip()
+        tokens = [context_msg, error_msg]
+        debug.printTokens(debug.LEVEL_WARNING, tokens, True)
+        _brlAPIConnectable = False
+
+    return _brlAPIConnectable
 
 def init(callback=None):
     """Initializes the braille module, connecting to the BrlTTY driver.
@@ -1849,6 +1912,9 @@ def init(callback=None):
     if not settings.enableBraille:
         return False
 
+    if not _canConnect():
+        return False
+
     global _brlAPI
     global _brlAPIRunning
     global _brlAPISourceId
@@ -1856,46 +1922,46 @@ def init(callback=None):
     global _callback
     global _monitor
 
-    msg = "BRAILLE: Initializing. Callback: %s" % callback
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: Initializing. Callback:", callback]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     if _brlAPIRunning:
         msg = "BRAILLE: BrlAPI is already running."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return True
 
     _callback = callback
 
-    msg = "BRAILLE: WINDOWPATH=%s" % os.environ.get("WINDOWPATH")
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: WINDOWPATH=", os.environ.get('WINDOWPATH')]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-    msg = "BRAILLE: XDG_VTNR=%s" % os.environ.get("XDG_VTNR")
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["BRAILLE: XDG_VTNR=", os.environ.get('XDG_VTNR')]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
     try:
         msg = "BRAILLE: Attempting connection with BrlAPI."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         _brlAPI = brlapi.Connection()
-        msg = "BRAILLE: Connection established with BrlAPI: %s" % _brlAPI
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["BRAILLE: Connection established with BrlAPI:", _brlAPI]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         msg = "BRAILLE: Attempting to enter TTY mode."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         _brlAPI.enterTtyModeWithPath()
         msg = "BRAILLE: TTY mode entered."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         _brlAPIRunning = True
 
         (x, y) = _brlAPI.displaySize
-        msg = "BRAILLE: Display size: (%i,%i)" % (x, y)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f"BRAILLE: Display size: ({x},{y})"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         if x == 0:
             msg = "BRAILLE: Error - 0 cells suggests display is not yet plugged in."
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
             raise Exception
 
         _brlAPISourceId = GLib.io_add_watch(_brlAPI.fileDescriptor,
@@ -1905,11 +1971,11 @@ def init(callback=None):
 
     except NameError:
         msg = "BRAILLE: Initialization failed: BrlApi is not defined."
-        debug.println(debug.LEVEL_WARNING, msg, True)
+        debug.printMessage(debug.LEVEL_WARNING, msg, True)
         return False
-    except:
+    except Exception:
         msg = "BRAILLE: Initialization failed."
-        debug.println(debug.LEVEL_WARNING, msg, True)
+        debug.printMessage(debug.LEVEL_WARNING, msg, True)
         debug.printException(debug.LEVEL_WARNING)
 
         _brlAPIRunning = False
@@ -1919,29 +1985,28 @@ def init(callback=None):
 
         try:
             msg = "BRAILLE: Attempting to leave TTY mode."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             _brlAPI.leaveTtyMode()
             msg = "BRAILLE: TTY mode exited."
-            debug.println(debug.LEVEL_INFO, msg, True)
-        except:
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        except Exception:
             msg = "BRAILLE: Exception leaving TTY mode."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         try:
             msg = "BRAILLE: Attempting to close connection."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             _brlAPI.closeConnection()
             msg = "BRAILLE: Connection closed."
-            debug.println(debug.LEVEL_INFO, msg, True)
-        except:
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+        except Exception:
             msg = "BRAILLE: Exception closing connection."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         _brlAPI = None
         return False
 
     _displaySize = [x, 1]
-    idle = False
 
     # The monitor will be created in refresh if needed.
     if _monitor:
@@ -1952,7 +2017,7 @@ def init(callback=None):
     refresh(True)
 
     msg = "BRAILLE: Initialized"
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     return True
 
 def shutdown():
@@ -1961,7 +2026,7 @@ def shutdown():
     """
 
     msg = "BRAILLE: Attempting braille shutdown."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     global _brlAPI
     global _brlAPIRunning
@@ -1973,32 +2038,32 @@ def shutdown():
         _brlAPIRunning = False
 
         msg = "BRAILLE: Removing BrlAPI Source ID."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         GLib.source_remove(_brlAPISourceId)
         _brlAPISourceId = 0
 
         try:
             msg = "BRAILLE: Attempting to leave TTY mode."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             _brlAPI.leaveTtyMode()
-        except:
+        except Exception:
             msg = "BRAILLE: Exception leaving TTY mode."
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
         else:
             msg = "BRAILLE: Leaving TTY mode succeeded."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         try:
             msg = "BRAILLE: Attempting to close connection."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             _brlAPI.closeConnection()
-        except:
+        except Exception:
             msg = "BRAILLE: Exception closing connection."
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
         else:
             msg = "BRAILLE: Closing connection succeeded."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         _brlAPI = None
 
@@ -2008,9 +2073,9 @@ def shutdown():
         _displaySize = [DEFAULT_DISPLAY_SIZE, 1]
     else:
         msg = "BRAILLE: Braille was not running."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return False
 
     msg = "BRAILLE: Braille shutdown complete."
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
     return True
