@@ -27,8 +27,11 @@ __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc." \
                 "Copyright (c) 2011-2016 Igalia, S.L."
 __license__   = "LGPL"
 
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
+
 import math
-import pyatspi
 import time
 from gi.repository import Gdk
 from gi.repository import GLib
@@ -37,9 +40,12 @@ from . import debug
 from . import keybindings
 from . import keynames
 from . import messages
+from . import orca
 from . import orca_state
 from . import script_manager
 from . import settings
+from .ax_object import AXObject
+from .ax_utilities import AXUtilities
 
 KEYBOARD_EVENT     = "keyboard"
 BRAILLE_EVENT      = "braille"
@@ -65,14 +71,14 @@ class InputEvent:
         pass
 
 def _getXkbStickyKeysState():
-    from subprocess import check_output, CalledProcessError
+    from subprocess import check_output
 
     try:
         output = check_output(['xkbset', 'q'])
         for line in output.decode('ASCII', errors='ignore').split('\n'):
             if line.startswith('Sticky-Keys = '):
                 return line.endswith('On')
-    except:
+    except Exception:
         pass
     return False
 
@@ -232,8 +238,8 @@ class KeyboardEvent(InputEvent):
         self.type = event.type
         self.hw_code = event.hw_code
         self.modifiers = event.modifiers & Gdk.ModifierType.MODIFIER_MASK
-        if event.modifiers & (1 << pyatspi.MODIFIER_NUMLOCK):
-            self.modifiers |= (1 << pyatspi.MODIFIER_NUMLOCK)
+        if event.modifiers & (1 << Atspi.ModifierType.NUMLOCK):
+            self.modifiers |= (1 << Atspi.ModifierType.NUMLOCK)
         self.event_string = event.event_string
         self.keyval_name = Gdk.keyval_name(event.id)
         if self.event_string  == "":
@@ -270,21 +276,23 @@ class KeyboardEvent(InputEvent):
         # trying to heuristically hack around this just by looking at the event
         # is not reliable. Ditto regarding asking Gdk for the numlock state.
         if self.keyval_name.startswith("KP"):
-            if event.modifiers & (1 << pyatspi.MODIFIER_NUMLOCK):
+            if event.modifiers & (1 << Atspi.ModifierType.NUMLOCK):
                 self._is_kp_with_numlock = True
 
         if self._script:
             self._app = self._script.app
             if not self._window:
-                self._window = orca_state.activeWindow = self._script.utilities.activeWindow()
-                msg = 'INPUT EVENT: Updated window and active window to %s' % self._window
-                debug.println(debug.LEVEL_INFO, msg, True)
+                orca.setActiveWindow(self._script.utilities.activeWindow())
+                self._window = orca_state.activeWindow
+                tokens = ["INPUT EVENT: Updated window and active window to", self._window]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        if self._window and self._app != self._window.getApplication():
-            self._script = script_manager.getManager().getScript(self._window.getApplication())
+        if self._window and self._app != AXObject.get_application(self._window):
+            self._script = script_manager.getManager().getScript(
+                AXObject.get_application(self._window))
             self._app = self._script.app
-            msg = 'INPUT EVENT: Updated script to %s' % self._script
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["INPUT EVENT: Updated script to", self._script]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         if self.is_duplicate:
             KeyboardEvent.duplicateCount += 1
@@ -293,13 +301,9 @@ class KeyboardEvent(InputEvent):
 
         self.keyType = None
 
-        _isPressed = event.type == pyatspi.KEY_PRESSED_EVENT
-
-        try:
-            role = self._obj.getRole()
-        except:
-            role = None
-        _mayEcho = _isPressed or role == pyatspi.ROLE_TERMINAL
+        _isPressed = event.type == Atspi.EventType.KEY_PRESSED_EVENT
+        role = AXObject.get_role(self._obj)
+        _mayEcho = _isPressed or role == Atspi.Role.TERMINAL
 
         if KeyboardEvent.stickyKeys and not self.isOrcaModifier() \
            and not KeyboardEvent.lastOrcaModifierAlone:
@@ -327,12 +331,13 @@ class KeyboardEvent(InputEvent):
         elif self.isModifierKey():
             self.keyType = KeyboardEvent.TYPE_MODIFIER
             self.shouldEcho = _mayEcho and settings.enableModifierKeys
-            if self.isOrcaModifier():
+            if self.isOrcaModifier() and not self.is_duplicate:
                 now = time.time()
                 if KeyboardEvent.lastOrcaModifierAlone:
                     if _isPressed:
                         KeyboardEvent.secondOrcaModifierTime = now
-                    if KeyboardEvent.secondOrcaModifierTime < KeyboardEvent.lastOrcaModifierAloneTime + 0.5:
+                    if (KeyboardEvent.secondOrcaModifierTime <
+                        KeyboardEvent.lastOrcaModifierAloneTime + 0.5):
                         # double-orca, let the real action happen
                         self._bypassOrca = True
                     if not _isPressed:
@@ -344,8 +349,10 @@ class KeyboardEvent(InputEvent):
                         KeyboardEvent.currentOrcaModifierAlone = True
                         KeyboardEvent.currentOrcaModifierAloneTime = now
                     else:
-                        KeyboardEvent.lastOrcaModifierAlone = KeyboardEvent.currentOrcaModifierAlone
-                        KeyboardEvent.lastOrcaModifierAloneTime = KeyboardEvent.currentOrcaModifierAloneTime
+                        KeyboardEvent.lastOrcaModifierAlone = \
+                            KeyboardEvent.currentOrcaModifierAlone
+                        KeyboardEvent.lastOrcaModifierAloneTime = \
+                            KeyboardEvent.currentOrcaModifierAloneTime
         elif self.isFunctionKey():
             self.keyType = KeyboardEvent.TYPE_FUNCTION
             self.shouldEcho = _mayEcho and settings.enableFunctionKeys
@@ -425,7 +432,7 @@ class KeyboardEvent(InputEvent):
         if self.is_duplicate:
             return
 
-        if self.type == pyatspi.KEY_RELEASED_EVENT:
+        if self.type == Atspi.EventType.KEY_RELEASED_EVENT:
             return
 
         if self._clickCount < 3:
@@ -454,25 +461,20 @@ class KeyboardEvent(InputEvent):
             keyval_name = self.keyval_name
             key_type = self.keyType
 
-        return ("KEYBOARD_EVENT:  type=%s\n" % self.type.value_name.upper()) \
-             + ("                 id=%s\n" % keyid) \
-             + ("                 hw_code=%s\n" % hw_code) \
-             + ("                 modifiers=%s\n" % modifiers) \
-             + ("                 event_string=(%s)\n" % event_string) \
-             + ("                 keyval_name=(%s)\n" % keyval_name) \
+        return (f"KEYBOARD_EVENT:  type={self.type.value_name.upper()}\n") \
+             + f"                 id={keyid}\n" \
+             + f"                 hw_code={hw_code}\n" \
+             + f"                 modifiers={modifiers}\n" \
+             + f"                 event_string=({event_string})\n" \
+             + f"                 keyval_name=({keyval_name})\n" \
              + ("                 timestamp=%d\n" % self.timestamp) \
-             + ("                 time=%f\n" % time.time()) \
-             + ("                 keyType=%s\n" % key_type) \
-             + ("                 clickCount=%s\n" % self._clickCount) \
-             + ("                 shouldEcho=%s\n" % self.shouldEcho)
+             + f"                 time={time.time():f}\n" \
+             + f"                 keyType={key_type}\n" \
+             + f"                 clickCount={self._clickCount}\n" \
+             + f"                 shouldEcho={self.shouldEcho}\n"
 
     def _shouldObscure(self):
-        try:
-            role = self._obj.getRole()
-        except:
-            return False
-
-        if role != pyatspi.ROLE_PASSWORD_TEXT:
+        if not AXUtilities.is_password_text(self._obj):
             return False
 
         if not self.isPrintableKey():
@@ -570,11 +572,11 @@ class KeyboardEvent(InputEvent):
             return self.keyType in KeyboardEvent.TYPE_LOCKING
 
         lockingKeys = ["Caps_Lock", "Shift_Lock", "Num_Lock", "Scroll_Lock"]
-        if not self.event_string in lockingKeys:
+        if self.event_string not in lockingKeys:
             return False
 
         if not orca_state.bypassNextCommand and not self._bypassOrca:
-            return not self.event_string in settings.orcaModifierKeys
+            return self.event_string not in settings.orcaModifierKeys
 
         return True
 
@@ -646,7 +648,7 @@ class KeyboardEvent(InputEvent):
     def isPressedKey(self):
         """Returns True if the key is pressed"""
 
-        return self.type == pyatspi.KEY_PRESSED_EVENT
+        return self.type == Atspi.EventType.KEY_PRESSED_EVENT
 
     def isPunctuationKey(self):
         """Return True if this is a punctuation key."""
@@ -683,9 +685,6 @@ class KeyboardEvent(InputEvent):
         if not self.isPrintableKey():
             return False
 
-        if orca_state.learnModeEnabled:
-            return False
-
         script = orca_state.activeScript
         return script and script.utilities.willEchoCharacter(self)
 
@@ -698,11 +697,11 @@ class KeyboardEvent(InputEvent):
             return None
 
         if self.event_string == "Caps_Lock":
-            mod = pyatspi.MODIFIER_SHIFTLOCK
+            mod = Atspi.ModifierType.SHIFTLOCK
         elif self.event_string == "Shift_Lock":
-            mod = pyatspi.MODIFIER_SHIFT
+            mod = Atspi.ModifierType.SHIFT
         elif self.event_string == "Num_Lock":
-            mod = pyatspi.MODIFIER_NUMLOCK
+            mod = Atspi.ModifierType.NUMLOCK
         else:
             return None
 
@@ -731,21 +730,26 @@ class KeyboardEvent(InputEvent):
 
         return self._obj
 
+    def getHandler(self):
+        """Returns the handler associated with this key event."""
+
+        return self._handler
+
     def _getUserHandler(self):
         # TODO - JD: This should go away once plugin support is in place.
         try:
             bindings = settings.keyBindingsMap.get(self._script.__module__)
-        except:
+        except Exception:
             bindings = None
         if not bindings:
             try:
                 bindings = settings.keyBindingsMap.get("default")
-            except:
+            except Exception:
                 bindings = None
 
         try:
             handler = bindings.getInputHandler(self)
-        except:
+        except Exception:
             handler = None
 
         return handler
@@ -779,20 +783,8 @@ class KeyboardEvent(InputEvent):
         if self._isReleaseForLastNonModifierKeyEvent():
             return scriptConsumes, 'Is release for last non-modifier keyevent'
 
-        if orca_state.learnModeEnabled:
-            if self.event_string == 'Escape':
-                self._consumer = self._script.exitLearnMode
-                return True, 'Exiting Learn Mode'
-
-            if self.event_string == 'F1' and not self.modifiers:
-                self._consumer = self._script.showHelp
-                return True, 'Showing Help'
-
-            if self.event_string in ['F2', 'F3'] and not self.modifiers:
-                self._consumer = self._script.listOrcaShortcuts
-                return True, 'Listing shortcuts'
-
-            self._consumer = self._presentHandler
+        if self._script.learnModePresenter.is_active():
+            self._consumer = self._script.learnModePresenter.handle_event
             return True, 'In Learn Mode'
 
         if self.isModifierKey():
@@ -800,10 +792,6 @@ class KeyboardEvent(InputEvent):
                 return False, 'Non-Orca modifier not in Learn Mode'
             return True, 'Orca modifier'
 
-        if orca_state.listNotificationsModeEnabled:
-            self._consumer = self._script.listNotifications
-            return True, 'Listing notifications'
-
         if not self._handler:
             return False, 'No handler'
 
@@ -827,16 +815,10 @@ class KeyboardEvent(InputEvent):
         if self.isPressedKey():
             self._script.presentationInterrupt()
 
-        return self._script.presentKeyboardEvent(self)
-
-    def _presentHandler(self, input_event=None):
-        if not self._handler:
+        if self._script.learnModePresenter.is_active():
             return False
 
-        if self._handler.learnModeEnabled and self._handler.description:
-            self._script.presentMessage(self._handler.description)
-
-        return True
+        return self._script.presentKeyboardEvent(self)
 
     def process(self):
         """Processes this input event."""
@@ -850,38 +832,38 @@ class KeyboardEvent(InputEvent):
         if self.is_duplicate:
             data = '%s DUPLICATE EVENT #%i' % (data, KeyboardEvent.duplicateCount)
 
-        msg = '\nvvvvv PROCESS %s: %s vvvvv' % (self.type.value_name.upper(), data)
-        debug.println(debug.LEVEL_INFO, msg, False)
+        msg = f'\nvvvvv PROCESS {self.type.value_name.upper()}: {data} vvvvv'
+        debug.printMessage(debug.LEVEL_INFO, msg, False)
 
-        msg = 'HOST_APP: %s' % self._app
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["HOST_APP:", self._app]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        msg = 'WINDOW:   %s' % self._window
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["WINDOW:", self._window]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        msg = 'LOCATION: %s' % self._obj
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["LOCATION:", self._obj]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        msg = 'CONSUME:  %s (%s)' % (self._should_consume, self._consume_reason)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["CONSUME:", self._should_consume, self._consume_reason]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         self._did_consume, self._result_reason = self._process()
 
         if self._should_consume != self._did_consume:
-            msg = 'CONSUMED: %s (%s)' % (self._did_consume, self._result_reason)
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CONSUMED:", self._did_consume, self._result_reason]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         if debug.LEVEL_INFO >= debug.debugLevel and orca_state.activeScript:
             attributes = orca_state.activeScript.getTransferableAttributes()
             for key, value in attributes.items():
-                msg = 'INPUT EVENT: %s: %s' % (key, value)
-                debug.println(debug.LEVEL_INFO, msg, True)
+                msg = f"INPUT EVENT: {key}: {value}"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-        msg = 'TOTAL PROCESSING TIME: %.4f' % (time.time() - startTime)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f"TOTAL PROCESSING TIME: {time.time() - startTime:.4f}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-        msg = '^^^^^ PROCESS %s: %s ^^^^^\n' % (self.type.value_name.upper(), data)
-        debug.println(debug.LEVEL_INFO, msg, False)
+        msg = f"^^^^^ PROCESS {self.type.value_name.upper()}: {data} ^^^^^\n"
+        debug.printMessage(debug.LEVEL_INFO, msg, False)
 
         return self._did_consume
 
@@ -891,7 +873,7 @@ class KeyboardEvent(InputEvent):
         if self._bypassOrca:
             if (self.event_string == "Caps_Lock" \
                 or self.event_string == "Shift_Lock") \
-               and self.type == pyatspi.KEY_PRESSED_EVENT:
+               and self.type == Atspi.EventType.KEY_PRESSED_EVENT:
                     self._lock_mod()
                     self.keyType = KeyboardEvent.TYPE_LOCKING
                     self._present()
@@ -941,52 +923,52 @@ class KeyboardEvent(InputEvent):
             def lockit():
                 try:
                     if modifiers & modifier:
-                        lock = pyatspi.KEY_UNLOCKMODIFIERS
-                        debug.println(debug.LEVEL_INFO, "Unlocking capslock", True)
+                        lock = Atspi.KeySynthType.UNLOCKMODIFIERS
+                        debug.printMessage(debug.LEVEL_INFO, "Unlocking capslock", True)
                     else:
-                        lock = pyatspi.KEY_LOCKMODIFIERS
-                        debug.println(debug.LEVEL_INFO, "Locking capslock", True)
-                    pyatspi.Registry.generateKeyboardEvent(modifier, None, lock)
-                    debug.println(debug.LEVEL_INFO, "Done with capslock", True)
-                except:
-                    debug.println(debug.LEVEL_INFO, "Could not trigger capslock, " \
+                        lock = Atspi.KeySynthType.LOCKMODIFIERS
+                        debug.printMessage(debug.LEVEL_INFO, "Locking capslock", True)
+                    Atspi.generate_keyboard_event(modifier, "", lock)
+                    debug.printMessage(debug.LEVEL_INFO, "Done with capslock", True)
+                except Exception:
+                    debug.printMessage(debug.LEVEL_INFO, "Could not trigger capslock, " \
                         "at-spi2-core >= 2.32 is needed for triggering capslock", True)
                     pass
             return lockit
         if self.event_string == "Caps_Lock":
-            modifier = 1 << pyatspi.MODIFIER_SHIFTLOCK
+            modifier = 1 << Atspi.ModifierType.SHIFTLOCK
         elif self.event_string == "Shift_Lock":
-            modifier = 1 << pyatspi.MODIFIER_SHIFT
+            modifier = 1 << Atspi.ModifierType.SHIFT
         else:
-            msg = "Unknown locking key %s" % self.event_string
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            tokens = ["Unknown locking key", self.event_string]
+            debug.printTokens(debug.LEVEL_WARNING, tokens, True)
             return
-        debug.println(debug.LEVEL_INFO, "Scheduling capslock", True)
+        debug.printMessage(debug.LEVEL_INFO, "Scheduling capslock", True)
         GLib.timeout_add(1, lock_mod(self.modifiers, modifier))
 
     def _consume(self):
         startTime = time.time()
         data = "'%s' (%d)" % (self.event_string, self.hw_code)
-        msg = 'vvvvv CONSUME %s: %s vvvvv' % (self.type.value_name.upper(), data)
-        debug.println(debug.LEVEL_INFO, msg, False)
+        msg = f'vvvvv CONSUME {self.type.value_name.upper()}: {data} vvvvv'
+        debug.printMessage(debug.LEVEL_INFO, msg, False)
 
         if self._consumer:
-            msg = 'INFO: Consumer is %s' % self._consumer.__name__
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f'INFO: Consumer is {self._consumer.__name__}'
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             self._consumer(self)
         elif self._handler.function:
-            msg = 'INFO: Handler is %s' % self._handler.description
-            debug.println(debug.LEVEL_INFO, msg, True)
+            msg = f'INFO: Handler is {self._handler.description}'
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             self._handler.function(self._script, self)
         else:
             msg = 'INFO: No handler or consumer'
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-        msg = 'TOTAL PROCESSING TIME: %.4f' % (time.time() - startTime)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = f'TOTAL PROCESSING TIME: {time.time() - startTime:.4f}'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-        msg = '^^^^^ CONSUME %s: %s ^^^^^' % (self.type.value_name.upper(), data)
-        debug.println(debug.LEVEL_INFO, msg, False)
+        msg = f'^^^^^ CONSUME {self.type.value_name.upper()}: {data} ^^^^^'
+        debug.printMessage(debug.LEVEL_INFO, msg, False)
 
         return False
 
@@ -1007,7 +989,7 @@ class MouseButtonEvent(InputEvent):
         display = Gdk.Display.get_default()
         seat = Gdk.Display.get_default_seat(display)
         _pointer = seat.get_pointer()
-    except:
+    except Exception:
         _pointer = None
 
     def __init__(self, event):
@@ -1045,9 +1027,11 @@ class MouseButtonEvent(InputEvent):
         if math.sqrt((self.x - x)**2 + (self.y - y)**2) < 25:
             return
 
-        msg = "WARNING: Event coordinates (%i, %i) may be bogus. " \
-              "Updating to (%i, %i)." % (self.x, self.y, x, y)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        msg = (
+            f"WARNING: Event coordinates ({self.x}, {self.y}) may be bogus. "
+            f"Updating to ({x}, {y})"
+        )
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         self.x, self.y = x, y
 
     def setClickCount(self):
@@ -1102,11 +1086,7 @@ class InputEventHandler:
         return (self.function == other.function)
 
     def processInputEvent(self, script, inputEvent):
-        """Processes an input event.  If learnModeEnabled is True,
-        this will merely present the description of the input event via
-        If learnModeEnabled is False, this will call the function bound
-        to this InputEventHandler instance, passing the inputEvent as
-        the sole argument to the function.
+        """Processes an input event.
 
         This function is expected to return True if it consumes the
         event; otherwise it is expected to return False.
@@ -1118,15 +1098,9 @@ class InputEventHandler:
         """
 
         consumed = False
-
-        if orca_state.learnModeEnabled and self._learnModeEnabled:
-            if self.description:
-                script.presentMessage(self.description)
-                consumed = True
-        else:
-            try:
-                consumed = self.function(script, inputEvent)
-            except:
-                debug.printException(debug.LEVEL_SEVERE)
+        try:
+            consumed = self.function(script, inputEvent)
+        except Exception:
+            debug.printException(debug.LEVEL_SEVERE)
 
         return consumed
