@@ -25,10 +25,11 @@ __copyright__ = "Copyright (c) 2011. Orca Team."
 __license__   = "LGPL"
 
 import importlib
-import pyatspi
 
 from . import debug
 from . import orca_state
+from .ax_object import AXObject
+from .ax_utilities import AXUtilities
 from .scripts import apps, toolkits
 
 class ScriptManager:
@@ -47,10 +48,8 @@ class ScriptManager:
              "orca.scripts.apps",
              "orca.scripts.toolkits"]
         self._appNames = \
-            {'Firefox': 'Mozilla',
-             'Icedove': 'Thunderbird',
+            {'Icedove': 'Thunderbird',
              'Nereid': 'Banshee',
-             'empathy-chat': 'empathy',
              'gnome-calculator': 'gcalctool',
              'gtk-window-decorator': 'switcher',
              'marco': 'switcher',
@@ -58,9 +57,10 @@ class ScriptManager:
              'metacity': 'switcher',
              'pluma': 'gedit',
             }
+        self._toolkitNames = \
+            {'WebKitGTK': 'WebKitGtk', 'GTK': 'gtk'}
 
         self.setActiveScript(None, "__init__")
-        self._desktop = pyatspi.Registry.getDesktop(0)
         self._active = False
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Initialized', True)
 
@@ -77,7 +77,7 @@ class ScriptManager:
     def deactivate(self):
         """Called when this script manager is deactivated."""
 
-        debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Dectivating', True)
+        debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Deactivating', True)
         if self._defaultScript:
             self._defaultScript.deregisterEventListeners()
         self._defaultScript = None
@@ -91,17 +91,17 @@ class ScriptManager:
     def getModuleName(self, app):
         """Returns the module name of the script to use for application app."""
 
-        try:
-            appAndNameExist = app is not None and app.name != ''
-        except (LookupError, RuntimeError):
-            appAndNameExist = False
-            msg = 'ERROR: %s no longer exists' % app
+        if app is None:
+            msg = 'ERROR: Cannot get module name for null app'
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return None
+
+        name = AXObject.get_name(app)
+        if not name:
+            msg = 'ERROR: Cannot get module name for nameless app'
             debug.println(debug.LEVEL_INFO, msg, True)
-
-        if not appAndNameExist:
             return None
 
-        name = app.name
         altNames = list(self._appNames.keys())
         if name.endswith(".py") or name.endswith(".bin"):
             name = name.split('.')[0]
@@ -118,32 +118,18 @@ class ScriptManager:
                     name = names[0]
                     break
 
-        msg = 'SCRIPT MANAGER: mapped %s to %s' % (app.name, name)
+        msg = f'SCRIPT MANAGER: mapped {AXObject.get_name(app)} to {name}'
         debug.println(debug.LEVEL_INFO, msg, True)
         return name
 
     def _toolkitForObject(self, obj):
         """Returns the name of the toolkit associated with obj."""
 
-        name = ''
-        if obj:
-            try:
-                attributes = obj.getAttributes()
-            except (LookupError, RuntimeError):
-                pass
-            else:
-                attrs = dict([attr.split(':', 1) for attr in attributes])
-                name = attrs.get('toolkit', '')
-
-        return name
+        name = AXObject.get_attribute(obj, 'toolkit')
+        return self._toolkitNames.get(name, name)
 
     def _scriptForRole(self, obj):
-        try:
-            role = obj.getRole()
-        except:
-            return ''
-
-        if role == pyatspi.ROLE_TERMINAL:
+        if AXUtilities.is_terminal(obj):
             return 'terminal'
 
         return ''
@@ -165,16 +151,16 @@ class ScriptManager:
             except OSError:
                 debug.examineProcesses()
 
-            debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Found %s' % moduleName, True)
+            debug.println(debug.LEVEL_INFO, f'SCRIPT MANAGER: Found {moduleName}', True)
             try:
                 if hasattr(module, 'getScript'):
                     script = module.getScript(app)
                 else:
                     script = module.Script(app)
                 break
-            except:
+            except Exception:
                 debug.printException(debug.LEVEL_INFO)
-                msg = 'ERROR: Could not load %s' % moduleName
+                msg = f'ERROR: Could not load {moduleName}'
                 debug.println(debug.LEVEL_INFO, msg, True)
 
         return script
@@ -192,14 +178,9 @@ class ScriptManager:
         if script:
             return script
 
-        try:
-            toolkitName = getattr(app, "toolkitName", None)
-        except (LookupError, RuntimeError):
-            msg = 'ERROR: Exception getting toolkitName for: %s' % app
-            debug.println(debug.LEVEL_INFO, msg, True)
-        else:
-            if app and toolkitName:
-                script = self._newNamedScript(app, toolkitName)
+        toolkitName = AXObject.get_application_toolkit_name(app)
+        if app and toolkitName:
+            script = self._newNamedScript(app, toolkitName)
 
         if not script:
             script = self.getDefaultScript(app)
@@ -224,27 +205,36 @@ class ScriptManager:
         if not self._active:
             return script
 
-        try:
-            appInDesktop = script.app in self._desktop
-        except:
-            appInDesktop = False
-
-        if appInDesktop:
+        if AXUtilities.is_application_in_desktop(script.app):
             return script
 
-        msg = "WARNING: %s is not in the registry's desktop" % script.app
-        debug.println(debug.LEVEL_INFO, msg, True)
-
         newScript = self._getScriptForAppReplicant(script.app)
         if newScript:
-            msg = "SCRIPT MANAGER: Script for app replicant found: %s" % newScript
-            debug.println(debug.LEVEL_INFO, msg, True)
             return newScript
 
-        msg = "WARNING: Failed to get a replacement script for %s" % script.app
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["WARNING: Failed to get a replacement script for", script.app]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return script
 
+    def getScriptForMouseButtonEvent(self, event):
+        isActive = AXUtilities.is_active(orca_state.activeWindow)
+        tokens = ["SCRIPT MANAGER:", orca_state.activeWindow, "is active:", isActive]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
+        if isActive and orca_state.activeScript:
+            return orca_state.activeScript
+
+        script = self.getDefaultScript()
+        activeWindow = script.utilities.activeWindow()
+        if not activeWindow:
+            return script
+
+        focusedObject = AXUtilities.get_focused_object(activeWindow)
+        if focusedObject:
+            return self.getScript(AXObject.get_application(focusedObject), focusedObject)
+
+        return self.getScript(AXObject.get_application(activeWindow), activeWindow)
+
     def getScript(self, app, obj=None, sanityCheck=False):
         """Get a script for an app (and make it if necessary).  This is used
         instead of a simple calls to Script's constructor.
@@ -285,7 +275,7 @@ class ScriptManager:
             else:
                 appScript = self._createScript(app, None)
                 self.appScripts[app] = appScript
-        except:
+        except Exception:
             msg = 'WARNING: Exception getting app script.'
             debug.printException(debug.LEVEL_ALL)
             debug.println(debug.LEVEL_WARNING, msg, True)
@@ -294,16 +284,9 @@ class ScriptManager:
         if customScript:
             return customScript
 
-        try:
-            role = obj.getRole()
-        except:
-            forceAppScript = False
-        else:
-            forceAppScript = role in [pyatspi.ROLE_FRAME, pyatspi.ROLE_STATUS_BAR]
-
         # Only defer to the toolkit script for this object if the app script
         # is based on a different toolkit.
-        if toolkitScript and not forceAppScript \
+        if toolkitScript and not (AXUtilities.is_frame(obj) or AXUtilities.is_status_bar(obj)) \
            and not issubclass(appScript.__class__, toolkitScript.__class__):
             return toolkitScript
 
@@ -338,35 +321,19 @@ class ScriptManager:
         if not self._active:
             return None
 
-        def _pid(app):
-            try:
-                result = app.get_process_id()
-                msg = "SCRIPT MANAGER: %s is has pid: %s" % (app, result)
-                debug.println(debug.LEVEL_INFO, msg, True)
-                return result
-            except:
-                msg = "SCRIPT MANAGER: Exception getting pid for %s" % app
-                debug.println(debug.LEVEL_INFO, msg, True)
-                return -1
-
-        def _isValidApp(app):
-            try:
-                result = app in self._desktop
-                msg = "SCRIPT MANAGER: %s is in desktop: %s" % (app, result)
-                debug.println(debug.LEVEL_INFO, msg, True)
-                return result
-            except:
-                msg = "SCRIPT MANAGER: Exception seeing if %s is in desktop" % app
-                debug.println(debug.LEVEL_INFO, msg, True)
-                return False
-
-        pid = _pid(app)
+        pid = AXObject.get_process_id(app)
         if pid == -1:
             return None
 
         items = self.appScripts.items()
         for a, script in items:
-            if a != app and _pid(a) == pid and _isValidApp(a):
+            if AXObject.get_process_id(a) != pid:
+                continue
+            if a != app and AXUtilities.is_application_in_desktop(a):
+                if script.app is None:
+                    script.app = a
+                tokens = ["SCRIPT MANAGER: Script for app replicant:", script, script.app]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 return script
 
         return None
@@ -376,36 +343,32 @@ class ScriptManager:
         deleting any scripts as necessary.
         """
 
-        appList = list(self.appScripts.keys())
-        try:
-            appList = [a for a in appList if a is not None and a not in self._desktop]
-        except:
-            debug.printException(debug.LEVEL_FINEST)
-            return
+        msg = "SCRIPT MANAGER: Checking and cleaning up scripts."
+        debug.println(debug.LEVEL_INFO, msg, True)
 
+        appList = list(self.appScripts.keys())
         for app in appList:
-            msg = "SCRIPT MANAGER: %s is no longer in registry's desktop" % app
-            debug.println(debug.LEVEL_INFO, msg, True)
+            if AXUtilities.is_application_in_desktop(app):
+                continue
 
             try:
                 appScript = self.appScripts.pop(app)
             except KeyError:
-                msg = "SCRIPT MANAGER: %s not found in appScripts" % app
-                debug.println(debug.LEVEL_INFO, msg, True)
+                tokens = ["SCRIPT MANAGER:", app, "not found in appScripts"]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 continue
 
-            msg = "SCRIPT MANAGER: Old script for app found: %s" % appScript
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["SCRIPT MANAGER: Old script for app found:", appScript, appScript.app]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
             newScript = self._getScriptForAppReplicant(app)
             if newScript:
-                msg = "SCRIPT MANAGER: Script for app replicant found: %s" % newScript
-                debug.println(debug.LEVEL_INFO, msg, True)
-
+                tokens = ["SCRIPT MANAGER: Transferring attributes:", newScript, newScript.app]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
                 attrs = appScript.getTransferableAttributes()
                 for attr, value in attrs.items():
-                    msg = "SCRIPT MANAGER: Setting %s to %s" % (attr, value)
-                    debug.println(debug.LEVEL_INFO, msg, True)
+                    tokens = ["SCRIPT MANAGER: Setting", attr, "to", value]
+                    debug.printTokens(debug.LEVEL_INFO, tokens, True)
                     setattr(newScript, attr, value)
 
             del appScript
