@@ -27,8 +27,10 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc." \
                 "Copyright (c) 2010-2013 The Orca Team."
 __license__   = "LGPL"
 
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 from gi.repository import Gtk
-import pyatspi
 
 import orca.cmdnames as cmdnames
 import orca.debug as debug
@@ -39,10 +41,10 @@ import orca.input_event as input_event
 import orca.messages as messages
 import orca.orca as orca
 import orca.orca_state as orca_state
-import orca.speech as speech
-import orca.settings as settings
 import orca.settings_manager as settings_manager
 import orca.structural_navigation as structural_navigation
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
 
 from .braille_generator import BrailleGenerator
 from .formatting import Formatting
@@ -99,7 +101,7 @@ class Script(default.Script):
         return Formatting(self)
 
     def getUtilities(self):
-        """Returns the utilites for this script."""
+        """Returns the utilities for this script."""
 
         return Utilities(self)
 
@@ -166,11 +168,6 @@ class Script(default.Script):
                 cmdnames.PAN_BRAILLE_RIGHT,
                 False) # Do not enable learn mode for this action
 
-        self.inputEventHandlers["whereAmISelectionHandler"] = \
-            input_event.InputEventHandler(
-                Script.whereAmISelection,
-                cmdnames.WHERE_AM_I_SELECTION)
-
     def getAppKeyBindings(self):
         """Returns the application-specific keybindings for this script."""
 
@@ -245,7 +242,7 @@ class Script(default.Script):
         tableFrame = Gtk.Frame()
         grid.attach(tableFrame, 0, 2, 1, 1)
 
-        label = Gtk.Label(label="<b>%s</b>" % guilabels.TABLE_NAVIGATION)
+        label = Gtk.Label(label=f"<b>{guilabels.TABLE_NAVIGATION}</b>")
         label.set_use_markup(True)
         tableFrame.set_label_widget(label)
 
@@ -293,31 +290,29 @@ class Script(default.Script):
         """Returns a dictionary with the app-specific preferences."""
 
         prefs = {
-            'speakCellSpan': self.speakCellSpanCheckButton.get_active(),
-            'speakCellHeaders': self.speakCellHeadersCheckButton.get_active(),
-            'skipBlankCells': self.skipBlankCellsCheckButton.get_active(),
-            'speakCellCoordinates': self.speakCellCoordinatesCheckButton.get_active(),
-            'speakSpreadsheetCoordinates': self.speakSpreadsheetCoordinatesCheckButton.get_active(),
-            'alwaysSpeakSelectedSpreadsheetRange': self.alwaysSpeakSelectedSpreadsheetRangeCheckButton.get_active(),
+            'speakCellSpan':
+                self.speakCellSpanCheckButton.get_active(),
+            'speakCellHeaders':
+                self.speakCellHeadersCheckButton.get_active(),
+            'skipBlankCells':
+                self.skipBlankCellsCheckButton.get_active(),
+            'speakCellCoordinates':
+                self.speakCellCoordinatesCheckButton.get_active(),
+            'speakSpreadsheetCoordinates':
+                self.speakSpreadsheetCoordinatesCheckButton.get_active(),
+            'alwaysSpeakSelectedSpreadsheetRange':
+                self.alwaysSpeakSelectedSpreadsheetRangeCheckButton.get_active(),
         }
 
         prefs.update(self.spellcheck.getPreferencesFromGUI())
         return prefs
 
-    def presentObject(self, obj, **args):
-        if not self._lastCommandWasStructNav:
-            super().presentObject(obj, **args)
-            return
-
-        utterances = self.speechGenerator.generateSpeech(obj, **args)
-        speech.speak(utterances)
-
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
         """
 
-        if self.flatReviewContext \
+        if self.flatReviewPresenter.is_active() \
            or not self.isBrailleBeginningShowing() \
            or self.utilities.isSpreadSheetCell(orca_state.locusOfFocus) \
            or not self.utilities.isTextArea(orca_state.locusOfFocus):
@@ -325,7 +320,7 @@ class Script(default.Script):
 
         text = orca_state.locusOfFocus.queryText()
         string, startOffset, endOffset = text.getTextAtOffset(
-            text.caretOffset, pyatspi.TEXT_BOUNDARY_LINE_START)
+            text.caretOffset, Atspi.TextBoundaryType.LINE_START)
         if 0 < startOffset:
             text.setCaretOffset(startOffset-1)
             return True
@@ -333,7 +328,7 @@ class Script(default.Script):
         obj = self.utilities.findPreviousObject(orca_state.locusOfFocus)
         try:
             text = obj.queryText()
-        except:
+        except Exception:
             pass
         else:
             orca.setLocusOfFocus(None, obj, notifyScript=False)
@@ -347,7 +342,7 @@ class Script(default.Script):
         entire document.
         """
 
-        if self.flatReviewContext \
+        if self.flatReviewPresenter.is_active() \
            or not self.isBrailleEndShowing() \
            or self.utilities.isSpreadSheetCell(orca_state.locusOfFocus) \
            or not self.utilities.isTextArea(orca_state.locusOfFocus):
@@ -355,7 +350,7 @@ class Script(default.Script):
 
         text = orca_state.locusOfFocus.queryText()
         string, startOffset, endOffset = text.getTextAtOffset(
-            text.caretOffset, pyatspi.TEXT_BOUNDARY_LINE_START)
+            text.caretOffset, Atspi.TextBoundaryType.LINE_START)
         if endOffset < text.characterCount:
             text.setCaretOffset(endOffset)
             return True
@@ -363,7 +358,7 @@ class Script(default.Script):
         obj = self.utilities.findNextObject(orca_state.locusOfFocus)
         try:
             text = obj.queryText()
-        except:
+        except Exception:
             pass
         else:
             orca.setLocusOfFocus(None, obj, notifyScript=False)
@@ -410,8 +405,9 @@ class Script(default.Script):
         """
 
         cell = orca_state.locusOfFocus
-        if cell and cell.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
-            cell = cell.parent
+        parent = AXObject.get_parent(cell)
+        if AXObject.get_role(parent) == Atspi.Role.TABLE_CELL:
+            cell = parent
 
         row, column, table = self.utilities.getRowColumnAndTable(cell)
         if table:
@@ -428,15 +424,16 @@ class Script(default.Script):
         """
 
         cell = orca_state.locusOfFocus
-        if cell and cell.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
-            cell = cell.parent
+        parent = AXObject.get_parent(cell)
+        if AXObject.get_role(parent) == Atspi.Role.TABLE_CELL:
+            cell = parent
 
         row, column, table = self.utilities.getRowColumnAndTable(cell)
         try:
             del self.dynamicColumnHeaders[hash(table)]
             self.presentationInterrupt()
             self.presentMessage(messages.DYNAMIC_COLUMN_HEADER_CLEARED)
-        except:
+        except Exception:
             pass
 
         return True
@@ -455,8 +452,9 @@ class Script(default.Script):
         """
 
         cell = orca_state.locusOfFocus
-        if cell and cell.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
-            cell = cell.parent
+        parent = AXObject.get_parent(cell)
+        if AXObject.get_role(parent) == Atspi.Role.TABLE_CELL:
+            cell = parent
 
         row, column, table = self.utilities.getRowColumnAndTable(cell)
         if table:
@@ -474,15 +472,16 @@ class Script(default.Script):
         """
 
         cell = orca_state.locusOfFocus
-        if cell and cell.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
-            cell = cell.parent
+        parent = AXObject.get_parent(cell)
+        if AXObject.get_role(parent) == Atspi.Role.TABLE_CELL:
+            cell = parent
 
         row, column, table = self.utilities.getRowColumnAndTable(cell)
         try:
             del self.dynamicRowHeaders[hash(table)]
             self.presentationInterrupt()
             self.presentMessage(messages.DYNAMIC_ROW_HEADER_CLEARED)
-        except:
+        except Exception:
             pass
 
         return True
@@ -505,12 +504,12 @@ class Script(default.Script):
             self.find()
             return
 
-        if self.flatReviewContext:
-            self.toggleFlatReviewMode()
+        if self.flatReviewPresenter.is_active():
+            self.flatReviewPresenter.quit()
 
         if self.spellcheck.isSuggestionsItem(newLocusOfFocus) \
            and not self.spellcheck.isSuggestionsItem(oldLocusOfFocus):
-            orca.emitRegionChanged(newFocus)
+            orca.emitRegionChanged(newLocusOfFocus)
             self.updateBraille(newLocusOfFocus)
             self.spellcheck.presentSuggestionListItem(includeLabel=True)
             return
@@ -519,27 +518,23 @@ class Script(default.Script):
         # If we are in the slide presentation scroll pane, also announce
         # the current page tab. See bug #538056 for more details.
         #
-        rolesList = [pyatspi.ROLE_SCROLL_PANE,
-                     pyatspi.ROLE_PANEL,
-                     pyatspi.ROLE_PANEL,
-                     pyatspi.ROLE_ROOT_PANE,
-                     pyatspi.ROLE_FRAME,
-                     pyatspi.ROLE_APPLICATION]
+        rolesList = [Atspi.Role.SCROLL_PANE,
+                     Atspi.Role.PANEL,
+                     Atspi.Role.PANEL,
+                     Atspi.Role.ROOT_PANE,
+                     Atspi.Role.FRAME,
+                     Atspi.Role.APPLICATION]
         if self.utilities.hasMatchingHierarchy(newLocusOfFocus, rolesList):
-            for child in newLocusOfFocus.parent:
-                if child.getRole() == pyatspi.ROLE_PAGE_TAB_LIST:
-                    for tab in child:
-                        eventState = tab.getState()
-                        if eventState.contains(pyatspi.STATE_SELECTED):
-                            utterances = \
-                                self.speechGenerator.generateSpeech(tab)
-                            speech.speak(utterances)
+            parent = AXObject.get_parent(newLocusOfFocus)
+            for child in AXObject.iter_children(parent, AXUtilities.is_page_tab_list):
+                for tab in AXObject.iter_children(child, AXUtilities.is_selected):
+                    self.presentObject(tab)
 
         # TODO - JD: This is a hack that needs to be done better. For now it
         # fixes the broken echo previous word on Return.
         elif newLocusOfFocus and oldLocusOfFocus \
-           and newLocusOfFocus.getRole() == pyatspi.ROLE_PARAGRAPH \
-           and oldLocusOfFocus.getRole() == pyatspi.ROLE_PARAGRAPH \
+           and AXObject.get_role(newLocusOfFocus) == Atspi.Role.PARAGRAPH \
+           and AXObject.get_role(oldLocusOfFocus) == Atspi.Role.PARAGRAPH \
            and newLocusOfFocus != oldLocusOfFocus:
             lastKey, mods = self.utilities.lastKeyAndModifiers()
             if lastKey == "Return" and _settingsManager.getSetting('enableEchoByWord'):
@@ -553,12 +548,14 @@ class Script(default.Script):
             isControlKey = mods & keybindings.CTRL_MODIFIER_MASK
             isShiftKey = mods & keybindings.SHIFT_MODIFIER_MASK
             if event_string in ["Up", "Down"] and isControlKey and not isShiftKey:
-                if self.utilities.displayedText(newLocusOfFocus):
-                    speech.speak(self.utilities.displayedText(newLocusOfFocus))
+                string = self.utilities.displayedText(newLocusOfFocus)
+                if string:
+                    voice = self.speechGenerator.voice(obj=newLocusOfFocus, string=string)
+                    self.speakMessage(string, voice=voice)
                     self.updateBraille(newLocusOfFocus)
                     try:
                         text = newLocusOfFocus.queryText()
-                    except:
+                    except Exception:
                         pass
                     else:
                         self._saveLastCursorPosition(newLocusOfFocus, text.caretOffset)
@@ -570,11 +567,15 @@ class Script(default.Script):
         if not newLocusOfFocus:
             return
 
+        parent = AXObject.get_parent(newLocusOfFocus)
+        if parent is None:
+            return
+
         cell = None
         if self.utilities.isTextDocumentCell(newLocusOfFocus):
             cell = newLocusOfFocus
-        elif self.utilities.isTextDocumentCell(newLocusOfFocus.parent):
-            cell = newLocusOfFocus.parent
+        elif self.utilities.isTextDocumentCell(parent):
+            cell = parent
         if cell:
             row, column = self.utilities.coordinatesForCell(cell)
             self.pointOfReference['lastRow'] = row
@@ -609,9 +610,9 @@ class Script(default.Script):
     def onActiveChanged(self, event):
         """Callback for object:state-changed:active accessibility events."""
 
-        if not event.source.parent:
+        if not AXObject.get_parent(event.source):
             msg = "SOFFICE: Event source lacks parent"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         # Prevent this events from activating the find operation.
@@ -633,7 +634,7 @@ class Script(default.Script):
             return
 
         if event.source == self.spellcheck.getSuggestionsList():
-            if event.source.getState().contains(pyatspi.STATE_FOCUSED):
+            if AXUtilities.is_focused(event.source):
                 orca.setLocusOfFocus(event, event.any_data, False)
                 self.updateBraille(orca_state.locusOfFocus)
                 self.spellcheck.presentSuggestionListItem()
@@ -641,6 +642,13 @@ class Script(default.Script):
                 self.spellcheck.presentErrorDetails()
             return
 
+        if self.utilities.isSpreadSheetCell(event.any_data) \
+           and not AXUtilities.is_focused(event.any_data) \
+           and not AXUtilities.is_focused(event.source) :
+            msg = "SOFFICE: Neither source nor child have focused state. Clearing cache on table."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            AXObject.clear_cache(event.source)
+
         default.Script.onActiveDescendantChanged(self, event)
 
     def onChildrenAdded(self, event):
@@ -686,37 +694,37 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
-        role = event.source.getRole()
+        role = AXObject.get_role(event.source)
 
         if self.utilities.isZombie(event.source) \
-           or role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_LIST]:
+           or role in [Atspi.Role.TEXT, Atspi.Role.LIST]:
             comboBox = self.utilities.containingComboBox(event.source)
             if comboBox:
                 orca.setLocusOfFocus(event, comboBox, True)
                 return
 
         # This seems to be something we inherit from Gtk+
-        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_PASSWORD_TEXT]:
+        if role in [Atspi.Role.TEXT, Atspi.Role.PASSWORD_TEXT]:
             orca.setLocusOfFocus(event, event.source)
             return
 
         # Ditto.
-        if role == pyatspi.ROLE_PUSH_BUTTON:
+        if role == Atspi.Role.PUSH_BUTTON:
             orca.setLocusOfFocus(event, event.source)
             return
 
         # Ditto.
-        if role == pyatspi.ROLE_TOGGLE_BUTTON:
+        if role == Atspi.Role.TOGGLE_BUTTON:
             orca.setLocusOfFocus(event, event.source)
             return
 
         # Ditto.
-        if role == pyatspi.ROLE_COMBO_BOX:
+        if role == Atspi.Role.COMBO_BOX:
             orca.setLocusOfFocus(event, event.source)
             return
 
         # Ditto.
-        if role == pyatspi.ROLE_PANEL and event.source.name:
+        if role == Atspi.Role.PANEL and AXObject.get_name(event.source):
             orca.setLocusOfFocus(event, event.source)
             return
 
@@ -734,38 +742,40 @@ class Script(default.Script):
 
         if self.utilities.isAnInputLine(event.source):
             msg = "SOFFICE: Event ignored: spam from inputLine"
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return
-        if event.source.childCount and self.utilities.isAnInputLine(event.source[0]):
-            msg = "SOFFICE: Event ignored: spam from inputLine parent"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
-        role = event.source.getRole()
-        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_LIST]:
+        if AXObject.get_child_count(event.source) \
+            and self.utilities.isAnInputLine(AXObject.get_child(event.source, 0)):
+            msg = "SOFFICE: Event ignored: spam from inputLine parent"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            return
+
+        role = AXObject.get_role(event.source)
+        if role in [Atspi.Role.TEXT, Atspi.Role.LIST]:
             comboBox = self.utilities.containingComboBox(event.source)
             if comboBox:
                 orca.setLocusOfFocus(event, comboBox, True)
                 return
 
-        parent = event.source.parent
-        if parent and parent.getRole() == pyatspi.ROLE_TOOL_BAR:
+        parent = AXObject.get_parent(event.source)
+        if parent and AXObject.get_role(parent) == Atspi.Role.TOOL_BAR:
             default.Script.onFocusedChanged(self, event)
             return
 
         # TODO - JD: Verify this is still needed
-        ignoreRoles = [pyatspi.ROLE_FILLER, pyatspi.ROLE_PANEL]
+        ignoreRoles = [Atspi.Role.FILLER, Atspi.Role.PANEL]
         if role in ignoreRoles:
             return
 
         # We will present this when the selection changes.
-        if role == pyatspi.ROLE_MENU:
+        if role == Atspi.Role.MENU:
             return
 
         if self.utilities._flowsFromOrToSelection(event.source):
             return
 
-        if role == pyatspi.ROLE_PARAGRAPH:
+        if role == Atspi.Role.PARAGRAPH:
             obj, offset = self.pointOfReference.get("lastCursorPosition", (None, -1))
             start, end, string = self.utilities.getCachedTextSelection(obj)
             if start != end:
@@ -779,13 +789,13 @@ class Script(default.Script):
         if self.utilities.isSpreadSheetTable(event.source) and orca_state.locusOfFocus:
             if self.utilities.isDead(orca_state.locusOfFocus):
                 msg = "SOFFICE: Event believed to be post-editing focus claim. Dead locusOfFocus."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 orca.setLocusOfFocus(event, event.source, False)
                 return
-
-            if  orca_state.locusOfFocus.getRole() in [pyatspi.ROLE_PARAGRAPH, pyatspi.ROLE_TABLE_CELL]:
+            if AXUtilities.is_paragraph(orca_state.locusOfFocus) \
+               or AXUtilities.is_table_cell(orca_state.locusOfFocus):
                 msg = "SOFFICE: Event believed to be post-editing focus claim based on role."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 orca.setLocusOfFocus(event, event.source, False)
                 return
 
@@ -797,6 +807,13 @@ class Script(default.Script):
         if event.detail1 == -1:
             return
 
+        if AXObject.get_role(event.source) == Atspi.Role.PARAGRAPH \
+           and not AXUtilities.is_focused(event.source):
+            AXObject.clear_cache(event.source)
+            if AXUtilities.is_focused(event.source):
+                msg = "SOFFICE: Clearing cache was needed due to missing state-changed event."
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
+
         if self.utilities._flowsFromOrToSelection(event.source):
            return
 
@@ -804,12 +821,12 @@ class Script(default.Script):
             return
 
         if self.utilities.isSpreadSheetCell(orca_state.locusOfFocus):
-            msg = "SOFFICE: locusOfFocus %s is spreadsheet cell" % orca_state.locusOfFocus
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["SOFFICE: locusOfFocus", orca_state.locusOfFocus, "is spreadsheet cell"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
             if not self.utilities.isCellBeingEdited(event.source):
                 msg = "SOFFICE: Event ignored: Source is not cell being edited."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 return
 
         super().onCaretMoved(event)
@@ -818,13 +835,13 @@ class Script(default.Script):
         """Callback for object:state-changed:checked accessibility events."""
 
         obj = event.source
-        role = obj.getRole()
-        parentRole = obj.parent.getRole()
-        if not role in [pyatspi.ROLE_TOGGLE_BUTTON, pyatspi.ROLE_PUSH_BUTTON] \
-           or not parentRole == pyatspi.ROLE_TOOL_BAR:
+        role = AXObject.get_role(obj)
+        parentRole = AXObject.get_role(AXObject.get_parent(obj))
+        if role not in [Atspi.Role.TOGGLE_BUTTON, Atspi.Role.PUSH_BUTTON] \
+           or not parentRole == Atspi.Role.TOOL_BAR:
             default.Script.onCheckedChanged(self, event)
             return
- 
+
         sourceWindow = self.utilities.topLevelObject(obj)
         focusWindow = self.utilities.topLevelObject(orca_state.locusOfFocus)
         if sourceWindow != focusWindow:
@@ -837,7 +854,7 @@ class Script(default.Script):
             x = orca_state.lastInputEvent.x
             y = orca_state.lastInputEvent.y
             weToggledIt = obj.queryComponent().contains(x, y, 0)
-        elif obj.getState().contains(pyatspi.STATE_FOCUSED):
+        elif AXUtilities.is_focused(obj):
             weToggledIt = True
         else:
             keyString, mods = self.utilities.lastKeyAndModifiers()
@@ -846,7 +863,7 @@ class Script(default.Script):
             wasCommand = mods & keybindings.COMMAND_MODIFIER_MASK
             weToggledIt = wasCommand and keyString not in navKeys
         if weToggledIt:
-            speech.speak(self.speechGenerator.generateSpeech(obj))
+            self.presentObject(obj, alreadyFocused=True, interrupt=True)
 
     def onSelectedChanged(self, event):
         """Callback for object:state-changed:selected accessibility events."""
@@ -854,11 +871,11 @@ class Script(default.Script):
         full, brief = "", ""
         if self.utilities.isSelectedTextDeletionEvent(event):
             msg = "SOFFICE: Change is believed to be due to deleting selected text"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             full = messages.SELECTION_DELETED
         elif self.utilities.isSelectedTextRestoredEvent(event):
             msg = "SOFFICE: Selection is believed to be due to restoring selected text"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             if self.utilities.handleUndoTextEvent(event):
                 full = messages.SELECTION_RESTORED
 
@@ -883,6 +900,18 @@ class Script(default.Script):
             self.utilities.handleCellSelectionChange(event.source)
             return
 
+        if event.source == self.spellcheck.getSuggestionsList():
+            if orca_state.locusOfFocus == orca_state.activeWindow:
+                msg = "SOFFICE: Not presenting because locusOfFocus is window"
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
+            elif AXUtilities.is_focused(event.source):
+                orca.setLocusOfFocus(event, event.any_data, False)
+                self.updateBraille(orca_state.locusOfFocus)
+                self.spellcheck.presentSuggestionListItem()
+            else:
+                self.spellcheck.presentErrorDetails()
+            return
+
         if not self.utilities.isComboBoxSelectionChange(event):
             super().onSelectionChanged(event)
             return
@@ -898,11 +927,16 @@ class Script(default.Script):
 
         if self.utilities.isComboBoxNoise(event):
             msg = "SOFFICE: Event is believed to be combo box noise"
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            return
+
+        if self.utilities.isDead(event.source):
+            msg = "SOFFICE: Ignoring event from dead source."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         if event.source != orca_state.locusOfFocus \
-           and event.source.getState().contains(pyatspi.STATE_FOCUSED):
+           and AXUtilities.is_focused(event.source):
             orca.setLocusOfFocus(event, event.source, False)
 
         super().onTextSelectionChanged(event)
@@ -910,7 +944,7 @@ class Script(default.Script):
     def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
         """To-be-removed. Returns the string, caretOffset, startOffset."""
 
-        if obj.parent.getRole() == pyatspi.ROLE_COMBO_BOX:
+        if AXObject.get_role(AXObject.get_parent(obj)) == Atspi.Role.COMBO_BOX:
             try:
                 text = obj.queryText()
             except NotImplementedError:
@@ -918,7 +952,7 @@ class Script(default.Script):
 
             if text.caretOffset < 0:
                 [lineString, startOffset, endOffset] = text.getTextAtOffset(
-                    0, pyatspi.TEXT_BOUNDARY_LINE_START)
+                    0, Atspi.TextBoundaryType.LINE_START)
 
                 # Sometimes we get the trailing line-feed -- remove it
                 #
@@ -928,7 +962,7 @@ class Script(default.Script):
                 return [lineString, 0, startOffset]
 
         textLine = super().getTextLineAtCaret(obj, offset, startOffset, endOffset)
-        if not obj.getState().contains(pyatspi.STATE_FOCUSED):
+        if not AXUtilities.is_focused(obj):
             textLine[0] = self.utilities.displayedText(obj)
 
         return textLine
@@ -940,8 +974,9 @@ class Script(default.Script):
         if not self.spellcheck.isCheckWindow(event.source):
             return
 
-        if event.source[0].getRole() == pyatspi.ROLE_DIALOG:
-            orca.setLocusOfFocus(event, event.source[0], False)
+        child = AXObject.get_child(event.source, 0)
+        if AXObject.get_role(child) == Atspi.Role.DIALOG:
+            orca.setLocusOfFocus(event, child, False)
 
         self.spellcheck.presentErrorDetails()
 
@@ -952,14 +987,3 @@ class Script(default.Script):
 
         super().onWindowDeactivated(event)
         self.spellcheck.deactivate()
-
-    def whereAmISelection(self, inputEvent=None, obj=None):
-        obj = obj or orca_state.locusOfFocus
-        if not self.utilities.isSpreadSheetCell(obj):
-            if self.utilities.inDocumentContent(obj):
-                # Because for some reason, the document implements the selection
-                # interface as if it were a spreadsheet or listbox. *sigh*
-                return super()._whereAmISelectedText(inputEvent, obj)
-            return super().whereAmISelection(inputEvent, obj)
-
-        return self.utilities.speakSelectedCellRange(self.utilities.getTable(obj))
