@@ -27,13 +27,16 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2018-2019 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 import re
-import time
 
 from orca import debug
 from orca import orca_state
 from orca.scripts import web
+from orca.ax_object import AXObject
+from orca.ax_utilities import AXUtilities
 
 
 class Utilities(web.Utilities):
@@ -56,14 +59,7 @@ class Utilities(web.Utilities):
         if not (obj and self.inDocumentContent(obj)):
             return super().isStaticTextLeaf(obj)
 
-        try:
-            childCount = obj.childCount
-        except:
-            msg = "CHROMIUM: Exception getting child count of %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return False
-
-        if childCount:
+        if AXObject.get_child_count(obj):
             return False
 
         if self.isListItemMarker(obj):
@@ -73,11 +69,11 @@ class Utilities(web.Utilities):
         if rv is not None:
             return rv
 
-        roles = [pyatspi.ROLE_STATIC, pyatspi.ROLE_TEXT]
-        rv = obj.getRole() in roles and self._getTag(obj) in (None, "br")
+        roles = [Atspi.Role.STATIC, Atspi.Role.TEXT]
+        rv = AXObject.get_role(obj) in roles and self._getTag(obj) in (None, "", "br")
         if rv:
-            msg = "CHROMIUM: %s believed to be static text leaf" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM:", obj, "believed to be static text leaf"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         self._isStaticTextLeaf[hash(obj)] = rv
         return rv
@@ -92,8 +88,8 @@ class Utilities(web.Utilities):
 
         rv = self._getTag(obj) in ["<pseudo:before>", "<pseudo:after>"]
         if rv:
-            msg = "CHROMIUM: %s believed to be pseudo element" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM:", obj, "believed to be pseudo element"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         self._isPseudoElement[hash(obj)] = rv
         return rv
@@ -107,248 +103,125 @@ class Utilities(web.Utilities):
             return rv
 
         rv = False
-        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_ITEM:
+        parent = AXObject.get_parent(obj)
+        if AXUtilities.is_list_item(parent):
             tag = self._getTag(obj)
             if tag == "::marker":
                 rv = True
-            elif tag is not None:
+            elif tag:
                 rv = False
-            elif obj.parent.childCount > 1:
-                rv = obj.parent[0] == obj
+            elif AXObject.get_child_count(parent) > 1:
+                rv = AXObject.get_child(parent, 0) == obj
             else:
-                rv = obj.name != self.displayedText(obj.parent)
+                rv = AXObject.get_name(obj) != self.displayedText(parent)
 
         self._isListItemMarker[hash(obj)] = rv
         return rv
 
-    def selectedChildCount(self, obj):
-        if not obj:
-            return []
-
-        count = super().selectedChildCount(obj)
-        if count or "Selection" in pyatspi.listInterfaces(obj):
-            return count
-
-        # HACK: Ideally, we'd use the selection interface to get the selected
-        # child count. But that interface is not implemented yet. This hackaround
-        # is extremely non-performant.
-        for child in obj:
-            if child.getState().contains(pyatspi.STATE_SELECTED):
-                count += 1
-
-        msg = "CHROMIUM: NO SELECTION INTERFACE HACK: Selected children: %i" % count
-        debug.println(debug.LEVEL_INFO, msg, True)
-        return count
-
-    def selectedChildren(self, obj):
-        if not obj:
-            return []
-
-        result = super().selectedChildren(obj)
-        if result or "Selection" in pyatspi.listInterfaces(obj):
-            return result
-
-        try:
-            childCount = obj.childCount
-        except:
-            msg = "CHROMIUM: Exception getting child count of %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return result
-
-        # HACK: Ideally, we'd use the selection interface to get the selected
-        # children. But that interface is not implemented yet. This hackaround
-        # is extremely non-performant.
-        for i in range(childCount):
-            child = obj[i]
-            if child and child.getState().contains(pyatspi.STATE_SELECTED):
-                result.append(child)
-
-        return result
-
-    def isMenuWithNoSelectedChild(self, obj):
-        if not obj:
-            return False
-
-        try:
-            role = obj.getRole()
-        except:
-            msg = "CHROMIUM: Exception getting role for %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return False
-
-        if role != pyatspi.ROLE_MENU:
-            return False
-
-        return not self.selectedChildCount(obj)
-
     def isMenuInCollapsedSelectElement(self, obj):
-        try:
-            role = obj.getRole()
-        except:
-            msg = "CHROMIUM: Exception getting role for %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+        if not AXUtilities.is_menu(obj):
             return False
 
-        if role != pyatspi.ROLE_MENU or self._getTag(obj.parent) != 'select':
+        parent = AXObject.get_parent(obj)
+        if self._getTag(parent) != 'select':
             return False
 
-        try:
-            parentState = obj.parent.getState()
-        except:
-            msg = "CHROMIUM: Exception getting state for %s" % obj.parent
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return False
-
-        return not parentState.contains(pyatspi.STATE_EXPANDED)
+        return not AXUtilities.is_expanded(parent)
 
     def treatAsMenu(self, obj):
-        if not obj:
-            return False
-
-        try:
-            role = obj.getRole()
-            state = obj.getState()
-        except:
-            msg = "CHROMIUM: Exception getting role and state for %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return False
-
         # Unlike other apps and toolkits, submenus in Chromium have the menu item
         # role rather than the menu role, but we can identify them as submenus via
         # the has-popup state.
-        if role == pyatspi.ROLE_MENU_ITEM:
-            return state.contains(pyatspi.STATE_HAS_POPUP)
-
-        return False
+        return AXUtilities.is_menu_item(obj) and AXUtilities.has_popup(obj)
 
     def isPopupMenuForCurrentItem(self, obj):
         # When a submenu is closed, it has role menu item. But when that submenu
         # is opened/expanded, a menu with that same name appears. It would be
         # nice if there were a connection (parent/child or an accessible relation)
         # between the two....
-        if not self.treatAsMenu(orca_state.locusOfFocus):
-            return False
-
-        if obj.name and obj.name == orca_state.locusOfFocus.name:
-            return obj.getRole() == pyatspi.ROLE_MENU
-
-        return False
+        return self.treatAsMenu(orca_state.locusOfFocus) and super().isPopupMenuForCurrentItem(obj)
 
     def isFrameForPopupMenu(self, obj):
-        try:
-            name = obj.name
-            role = obj.getRole()
-            childCount = obj.childCount
-        except:
-            msg = "CHROMIUM: Exception getting properties of %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return False
-
         # The ancestry of a popup menu appears to be a menu bar (even though
         # one is not actually showing) contained in a nameless frame. It would
         # be nice if these things were pruned from the accessibility tree....
-        if name or role != pyatspi.ROLE_FRAME or childCount != 1:
+        if not AXUtilities.is_frame(obj):
             return False
-
-        if obj[0].getRole() == pyatspi.ROLE_MENU_BAR:
-            return True
-
-        return False
+        if AXObject.get_name(obj):
+            return False
+        if AXObject.get_child_count(obj) != 1:
+            return False
+        return AXUtilities.is_menu_bar(AXObject.get_child(obj, 0))
 
     def isTopLevelMenu(self, obj):
-        if obj.getRole() == pyatspi.ROLE_MENU:
-            return self.isFrameForPopupMenu(self.topLevelObject(obj))
-
-        return False
+        return AXUtilities.is_menu(obj) and self.isFrameForPopupMenu(self.topLevelObject(obj))
 
     def popupMenuForFrame(self, obj):
         if not self.isFrameForPopupMenu(obj):
             return None
 
-        try:
-            menu = pyatspi.findDescendant(obj, lambda x: x and x.getRole() == pyatspi.ROLE_MENU)
-        except:
-            msg = "CHROMIUM: Exception finding descendant of %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            return None
-
-        msg = "CHROMIUM: HACK: Popup menu for %s: %s" % (obj, menu)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        menu = AXObject.find_descendant(obj, AXUtilities.is_menu)
+        tokens = ["CHROMIUM: HACK: Popup menu for", obj, ":", menu]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         return menu
 
-    def _getActionNames(self, obj):
-        names = super()._getActionNames(obj)
-
-        # click-ancestor is meant to bubble up to an ancestor which is actually
-        # clickable. But attempting to perform this action doesn't reliably work;
-        # and the clickable ancestor is what we want to click on anyway. Treating
-        # this as a valid action is causing us to include otherwise ignorable
-        # elements such as sections with no semantic meaning.
-        if "click-ancestor" in names:
-            names = list(filter(lambda x: x != "click-ancestor", names))
-            msg = "CHROMIUM: Ignoring 'click-ancestor' action on %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-
-        return names
-
-    def topLevelObject(self, obj):
+    def topLevelObject(self, obj, useFallbackSearch=False):
         if not obj:
             return None
 
         result = super().topLevelObject(obj)
-        if result and result.getRole() in self._topLevelRoles():
+        if AXObject.get_role(result) in self._topLevelRoles():
             if not self.isFindContainer(result):
                 return result
             else:
-                msg = "CHROMIUM: Top level object for %s is %s" % (obj, result.parent)
-                debug.println(debug.LEVEL_INFO, msg, True)
-                return result.parent
+                parent = AXObject.get_parent(result)
+                tokens = ["CHROMIUM: Top level object for", obj, "is", parent]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+                return parent
 
         cached = self._topLevelObject.get(hash(obj))
         if cached is not None:
             return cached
 
-        msg = "CHROMIUM: WARNING: Top level object for %s is %s" % (obj, result)
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["CHROMIUM: WARNING: Top level object for", obj, "is", result]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         # The only (known) object giving us a broken ancestry is the omnibox popup.
-        roles = [pyatspi.ROLE_LIST_ITEM, pyatspi.ROLE_LIST_BOX]
-        if not (obj and obj.getRole() in roles):
+        if not (AXUtilities.is_list_item(obj or AXUtilities.is_list_box(obj))):
             return result
 
         listbox = obj
-        if obj.getRole() == pyatspi.ROLE_LIST_ITEM:
-            listbox = listbox.parent
+        if AXUtilities.is_list_item(obj):
+            listbox = AXObject.get_parent(listbox)
 
-        if not listbox:
+        if listbox is None:
             return result
 
         # The listbox sometimes claims to be a redundant object rather than a listbox.
         # Clearing the AT-SPI2 cache seems to be the trigger.
-        if not (listbox and listbox.getRole() in roles):
-            if listbox.getRole() == pyatspi.ROLE_REDUNDANT_OBJECT:
-                msg = "CHROMIUM: WARNING: Suspected bogus role on listbox %s" % listbox
-                debug.println(debug.LEVEL_INFO, msg, True)
+        if not AXUtilities.is_list_box(listbox):
+            if AXUtilities.is_redundant_object(listbox):
+                tokens = ["CHROMIUM: WARNING: Suspected bogus role on listbox", listbox]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
             else:
                 return result
 
         autocomplete = self.autocompleteForPopup(listbox)
         if autocomplete:
             result = self.topLevelObject(autocomplete)
-            msg = "CHROMIUM: Top level object for %s is %s" % (autocomplete, result)
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM: Top level object for", autocomplete, "is", result]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         self._topLevelObject[hash(obj)] = result
         return result
 
     def autocompleteForPopup(self, obj):
-        popupFor = lambda r: r.getRelationType() == pyatspi.RELATION_POPUP_FOR
-        relations = list(filter(popupFor, obj.getRelationSet()))
-        if not relations:
+        relation = AXObject.get_relation(obj, Atspi.RelationType.POPUP_FOR)
+        if not relation:
             return None
 
-        target = relations[0].getTarget(0)
-        if target and target.getRole() == pyatspi.ROLE_AUTOCOMPLETE:
+        target = relation.get_target(0)
+        if AXUtilities.is_autocomplete(target):
             return target
 
         return None
@@ -360,7 +233,7 @@ class Utilities(web.Utilities):
         return self.autocompleteForPopup(obj) is not None
 
     def isRedundantAutocompleteEvent(self, event):
-        if event.source.getRole() != pyatspi.ROLE_AUTOCOMPLETE:
+        if not AXUtilities.is_autocomplete(event.source):
             return False
 
         if event.type.startswith("object:text-caret-moved"):
@@ -373,11 +246,10 @@ class Utilities(web.Utilities):
     def setCaretPosition(self, obj, offset, documentFrame=None):
         super().setCaretPosition(obj, offset, documentFrame)
 
-        isLink = lambda x: x and x.getRole() == pyatspi.ROLE_LINK
-        link = pyatspi.utils.findAncestor(obj, isLink)
-        if link:
-            msg = "CHROMIUM: HACK: Grabbing focus on %s's ancestor %s" % (obj, link)
-            debug.println(debug.LEVEL_INFO, msg, True)
+        link = AXObject.find_ancestor(obj, AXUtilities.is_link)
+        if link is not None:
+            tokens = ["CHROMIUM: HACK: Grabbing focus on", obj, "'s ancestor", link]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             self.grabFocus(link)
 
     def handleAsLiveRegion(self, event):
@@ -393,12 +265,11 @@ class Utilities(web.Utilities):
         # additions are not always coming to us in presentational order, whereas
         # the text changes appear to be. So most of the time, we can ignore the
         # children-changed events. Except for when we can't.
-
-        if event.any_data.getRole() == pyatspi.ROLE_TABLE:
+        if AXUtilities.is_table(event.any_data):
             return True
 
         msg = "CHROMIUM: Event is believed to be redundant live region notification"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return False
 
     def getFindResultsCount(self, root=None):
@@ -406,15 +277,14 @@ class Utilities(web.Utilities):
         if not root:
             return ""
 
-        isMatch = lambda x: x and x.getRole() == pyatspi.ROLE_STATUS_BAR
-        statusBars = self.findAllDescendants(root, isMatch)
+        statusBars = AXUtilities.find_all_status_bars(root)
         if len(statusBars) != 1:
             return ""
 
         bar = statusBars[0]
-        bar.clearCache()
-        if len(re.findall("\d+", bar.name)) == 2:
-            return bar.name
+        AXObject.clear_cache(bar)
+        if len(re.findall(r"\d+", AXObject.get_name(bar))) == 2:
+            return AXObject.get_name(bar)
 
         return ""
 
@@ -425,13 +295,13 @@ class Utilities(web.Utilities):
         if obj == self._findContainer:
             return True
 
-        if obj.getRole() != pyatspi.ROLE_DIALOG:
+        if not AXUtilities.is_dialog(obj):
             return False
 
         result = self.getFindResultsCount(obj)
         if result:
-            msg = "CHROMIUM: %s believed to be find-in-page container (%s)" % (obj, result)
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM:", obj, "believed to be find-in-page container (", result, ")"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             self._findContainer = obj
             return True
 
@@ -441,44 +311,38 @@ class Utilities(web.Utilities):
         # back on the widgets. TODO: This would be far easier if Chromium gave us an
         # object attribute we could look for....
 
-        isEntry = lambda x: x.getRole() == pyatspi.ROLE_ENTRY
-        if len(self.findAllDescendants(obj, isEntry)) != 1:
-            msg = "CHROMIUM: %s not believed to be find-in-page container (entry count)" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+        if len(AXUtilities.find_all_entries(obj)) != 1:
+            tokens = ["CHROMIUM:", obj, "not believed to be find-in-page container (entry count)"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return False
 
-        isButton = lambda x: x.getRole() == pyatspi.ROLE_PUSH_BUTTON
-        if len(self.findAllDescendants(obj, isButton)) != 3:
-            msg = "CHROMIUM: %s not believed to be find-in-page container (button count)" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+        if len(AXUtilities.find_all_push_buttons(obj)) != 3:
+            tokens = ["CHROMIUM:", obj, "not believed to be find-in-page container (button count)"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return False
 
-        isSeparator = lambda x: x.getRole() == pyatspi.ROLE_SEPARATOR
-        if len(self.findAllDescendants(obj, isSeparator)) != 1:
-            msg = "CHROMIUM: %s not believed to be find-in-page container (separator count)" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+        if len(AXUtilities.find_all_separators(obj)) != 1:
+            tokens = ["CHROMIUM:", obj,
+                      "not believed to be find-in-page container (separator count)"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return False
 
-        msg = "CHROMIUM: %s believed to be find-in-page container (accessibility tree)" % obj
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["CHROMIUM:", obj, "believed to be find-in-page container (accessibility tree)"]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         self._findContainer = obj
         return True
 
     def inFindContainer(self, obj=None):
-        if not obj:
-            obj = orca_state.locusOfFocus
-
-        if not obj or self.inDocumentContent(obj):
+        obj = obj or orca_state.locusOfFocus
+        if not (AXUtilities.is_entry(obj) or AXUtilities.is_push_button(obj)):
+            return False
+        if self.inDocumentContent(obj):
             return False
 
-        if obj.getRole() not in [pyatspi.ROLE_ENTRY, pyatspi.ROLE_PUSH_BUTTON]:
-            return False
-
-        isDialog = lambda x: x and x.getRole() == pyatspi.ROLE_DIALOG
-        result = self.isFindContainer(pyatspi.findAncestor(obj, isDialog))
+        result = self.isFindContainer(AXObject.find_ancestor(obj, AXUtilities.is_dialog))
         if result:
-            msg = "CHROMIUM: %s believed to be find-in-page widget" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM:", obj, "believed to be find-in-page widget"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         return result
 
@@ -491,9 +355,6 @@ class Utilities(web.Utilities):
 
         return True
 
-    def supportsLandmarkRole(self):
-        return True
-
     def findAllDescendants(self, root, includeIf=None, excludeIf=None):
         if not root:
             return []
@@ -501,8 +362,8 @@ class Utilities(web.Utilities):
         # Don't bother if the root is a 'pre' or 'code' element. Those often have
         # nothing but a TON of static text leaf nodes, which we want to ignore.
         if self._getTag(root) in ('pre', 'code'):
-            msg = "CHROMIUM: Returning 0 descendants for pre/code %s" % root
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM: Returning 0 descendants for pre/code", root]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return []
 
         return super().findAllDescendants(root, includeIf, excludeIf)
@@ -516,7 +377,7 @@ class Utilities(web.Utilities):
         # the point is still within its bounds. Therefore, we need to call
         # accessibleAtPoint() twice to be safe.
         msg = "CHROMIUM: Getting accessibleAtPoint again due to async hit test result."
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         result = super().accessibleAtPoint(root, x, y, coordType)
         return result
 
@@ -524,7 +385,7 @@ class Utilities(web.Utilities):
         if super()._isActiveAndShowingAndNotIconified(obj):
             return True
 
-        if obj and obj.getApplication() != self._script.app:
+        if obj and AXObject.get_application(obj) != self._script.app:
             return False
 
         # FIXME: This can potentially be non-performant because AT-SPI2 will recursively
@@ -532,17 +393,42 @@ class Utilities(web.Utilities):
         # be a lack of window:activate and object:state-changed events from Chromium
         # windows in at least some environments.
         try:
-            msg = "CHROMIUM: Clearing cache for %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
-            obj.clearCache()
-        except:
-            msg = "CHROMIUM: Exception clearing cache for %s" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM: Clearing cache for", obj]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            AXObject.clear_cache(obj)
+        except Exception:
+            tokens = ["CHROMIUM: Exception clearing cache for", obj]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return False
 
         if super()._isActiveAndShowingAndNotIconified(obj):
-            msg = "CHROMIUM: %s deemed to be active and showing after cache clear" % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["CHROMIUM:", obj, "deemed to be active and showing after cache clear"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return True
 
         return False
+
+    def _shouldCalculatePositionAndSetSize(self, obj):
+        # Chromium calculates posinset and setsize for description lists based on the
+        # number of terms present. If we want to present the number of values associated
+        # with a given term, we need to work those values out ourselves.
+        if self.isDescriptionListDescription(obj):
+            return True
+
+        if self.inDocumentContent(obj):
+            return super()._shouldCalculatePositionAndSetSize(obj)
+
+        # Chromium has accessible menu items which are not focusable and therefore do not
+        # have a posinset and setsize calculated. But they may claim to be the selected
+        # item when an accessible child is selected (e.g. "zoom" when "+" or "-" gains focus.
+        # Normally we calculate posinset and setsize when the application hasn't provided it.
+        # We don't want to do that in the case of menu items like "zoom" because our result
+        # will not jibe with the values of its siblings. Thus if a sibling has a value,
+        # assume that the missing attributes are missing on purpose.
+        for sibling in AXObject.iter_children(AXObject.get_parent(obj)):
+            if self.getPositionInSet(sibling) is not None:
+                tokens = ["CHROMIUM:", obj, "'s sibling", sibling, "has posinset."]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+                return False
+
+        return True
