@@ -27,8 +27,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import pyatspi
-
 from . import messages
 from .acss import ACSS
 
@@ -128,6 +126,7 @@ userCustomizableSettings = [
     "structNavTriggersFocusMode",
     "caretNavTriggersFocusMode",
     "layoutMode",
+    "nativeNavTriggersFocusMode",
     "rewindAndFastForwardInSayAll",
     "structNavInSayAll",
     "speakDescription",
@@ -255,7 +254,6 @@ sayAllContextList            = True
 sayAllContextTable           = True
 
 # Braille
-tty = 7
 enableBraille                  = True
 enableBrailleMonitor           = False
 enableBrailleContext           = True
@@ -392,6 +390,7 @@ timeoutCallback         = None # Set by orca.py:init to orca.timeout
 
 structNavTriggersFocusMode = False
 caretNavTriggersFocusMode = False
+nativeNavTriggersFocusMode = True
 
 layoutMode = True
 
