@@ -28,7 +28,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010 Consorcio Fernando de los Rios."
 __license__   = "LGPL"
 
-import imp
 import importlib
 import os
 from gi.repository import Gio, GLib
@@ -39,6 +38,7 @@ from . import script_manager
 from . import settings
 from . import pronunciation_dict
 from .acss import ACSS
+from .ax_object import AXObject
 from .keybindings import KeyBinding
 
 try:
@@ -50,7 +50,7 @@ try:
         '/org/a11y/bus',
         'org.freedesktop.DBus.Properties',
         None)
-except:
+except Exception:
     _proxy = None
 
 _scriptManager = script_manager.getManager()
@@ -73,7 +73,7 @@ class SettingsManager(object):
         backend='json'
         """
 
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initializing', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initializing', True)
 
         self.backendModule = None
         self._backend = None
@@ -116,10 +116,10 @@ class SettingsManager(object):
         # For handling the currently-"classic" application settings
         self.settingsPackages = ["app-settings"]
 
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initialized', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Initialized', True)
 
     def activate(self, prefsDir=None, customSettings={}):
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activating', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activating', True)
 
         self.customizedSettings.update(customSettings)
         self._prefsDir = prefsDir \
@@ -142,15 +142,15 @@ class SettingsManager(object):
         #
         self._createDefaults()
 
-        debug.println(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activated', True)
+        debug.printMessage(debug.LEVEL_INFO, 'SETTINGS MANAGER: Activated', True)
 
         # Set the active profile and load its stored settings
-        msg = 'SETTINGS MANAGER: Current profile is %s' % self.profile
+        msg = f'SETTINGS MANAGER: Current profile is {self.profile}'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         if self.profile is None:
             self.profile = self.general.get('startingProfile')[1]
-            msg = 'SETTINGS MANAGER: Current profile is now %s' % self.profile
+            msg = f'SETTINGS MANAGER: Current profile is now {self.profile}'
             debug.println(debug.LEVEL_INFO, msg, True)
 
         self.setProfile(self.profile)
@@ -159,10 +159,10 @@ class SettingsManager(object):
         """Load specific backend for manage user settings"""
 
         try:
-            backend = '.backends.%s_backend' % self.backendName
+            backend = f'.backends.{self.backendName}_backend'
             self.backendModule = importlib.import_module(backend, 'orca')
             return True
-        except:
+        except Exception:
             return False
 
     def _createDefaults(self):
@@ -225,7 +225,7 @@ class SettingsManager(object):
             if value is None:
                 try:
                     value = getattr(settings, key)
-                except:
+                except Exception:
                     pass
             self.defaultGeneral[key] = value
 
@@ -251,21 +251,25 @@ class SettingsManager(object):
 
         success = False
         pathList = [self._prefsDir]
+        msg = "SETTINGS MANAGER: Attempt to load orca-customizations "
+        module_path = pathList[0] + "/orca-customizations.py"
+
         try:
-            msg = "SETTINGS MANAGER: Attempt to load orca-customizations "
-            (fileHnd, moduleName, desc) = \
-                imp.find_module("orca-customizations", pathList)
-            msg += "from %s " % moduleName
-            imp.load_module("orca-customizations", fileHnd, moduleName, desc)
-        except ImportError:
-            success = True
-            msg += "failed due to ImportError. Giving up."
-        except AttributeError:
-            return False
-        else:
-            msg += "succeeded."
-            fileHnd.close()
+            spec = importlib.util.spec_from_file_location("orca-customizations", module_path)
+            if spec is not None:
+                module = importlib.util.module_from_spec(spec)
+                spec.loader.exec_module(module)
+                msg += f"from {module_path} succeeded."
+                success = True
+            else:
+                msg += f"from {module_path} failed. Spec not found."
+        except FileNotFoundError:
+            msg += f"from {module_path} failed. File not found."
+        except Exception as error:
+            # Treat this failure as a "success" so that we don't stomp on the existing file.
             success = True
+            msg += f"failed due to: {error}. Not loading customizations."
+
         debug.println(debug.LEVEL_ALL, msg, True)
         return success
 
@@ -284,7 +288,7 @@ class SettingsManager(object):
         lang = v.getLocale()
         dialect = v.getDialect()
         if dialect and len(str(dialect)) == 2:
-            lang = "%s_%s" % (lang, dialect.upper())
+            lang = f"{lang}_{dialect.upper()}"
         return lang
 
     def getSpeechServerFactories(self):
@@ -293,13 +297,13 @@ class SettingsManager(object):
         factories = []
         for moduleName in self.getSetting('speechFactoryModules'):
             try:
-                module = importlib.import_module('orca.%s' % moduleName)
+                module = importlib.import_module(f'orca.{moduleName}')
                 factories.append(module)
-                msg = "SETTINGS MANAGER: Valid speech server factory: %s" % moduleName
-                debug.println(debug.LEVEL_INFO, msg, True)
-            except:
-                msg = "SETTINGS MANAGER: Invalid speech server factory: %s" % moduleName
-                debug.println(debug.LEVEL_INFO, msg, True)
+                tokens = ["SETTINGS MANAGER: Valid speech server factory:", moduleName]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            except Exception:
+                tokens = ["SETTINGS MANAGER: Invalid speech server factory:", moduleName]
+                debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
         return factories
 
@@ -309,7 +313,7 @@ class SettingsManager(object):
         A profile can be passed as a parameter. This could be useful for
         change from one profile to another."""
 
-        msg = 'SETTINGS MANAGER: Loading settings for %s profile' % profile
+        msg = f'SETTINGS MANAGER: Loading settings for {profile} profile'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         if profile is None:
@@ -318,7 +322,7 @@ class SettingsManager(object):
         self.profilePronunciations = self.getPronunciations(profile) or {}
         self.profileKeybindings = self.getKeybindings(profile) or {}
 
-        msg = 'SETTINGS MANAGER: Settings for %s profile loaded' % profile
+        msg = f'SETTINGS MANAGER: Settings for {profile} profile loaded'
         debug.println(debug.LEVEL_INFO, msg, True)
 
     def _mergeSettings(self):
@@ -326,7 +330,7 @@ class SettingsManager(object):
         over the current and active settings"""
 
         msg = 'SETTINGS MANAGER: Merging settings.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         self.profileGeneral.update(self._appGeneral)
         self.profilePronunciations.update(self._appPronunciations)
@@ -337,7 +341,7 @@ class SettingsManager(object):
         self.keybindings.update(self.profileKeybindings)
 
         msg = 'SETTINGS MANAGER: Settings merged.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _enableAccessibility(self):
         """Enables the GNOME accessibility flag.  Users need to log out and
@@ -355,7 +359,7 @@ class SettingsManager(object):
 
     def isAccessibilityEnabled(self):
         msg = 'SETTINGS MANAGER: Checking if accessibility is enabled.'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
         msg = 'SETTINGS MANAGER: Accessibility enabled: '
         if not _proxy:
@@ -369,18 +373,18 @@ class SettingsManager(object):
         return rv
 
     def setAccessibility(self, enable):
-        msg = 'SETTINGS MANAGER: Attempting to set accessibility to %s.' % enable
+        msg = f'SETTINGS MANAGER: Attempting to set accessibility to {enable}.'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         if not _proxy:
             msg = 'SETTINGS MANAGER: Error (no proxy)'
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return False
 
         vEnable = GLib.Variant('b', enable)
         _proxy.Set('(ssv)', 'org.a11y.Status', 'IsEnabled', vEnable)
 
-        msg = 'SETTINGS MANAGER: Finished setting accessibility to %s.' % enable
+        msg = f'SETTINGS MANAGER: Finished setting accessibility to {enable}.'
         debug.println(debug.LEVEL_INFO, msg, True)
 
     def isScreenReaderServiceEnabled(self):
@@ -413,7 +417,7 @@ class SettingsManager(object):
         Also the settings from that profile will be loading
         and updated the current settings with them."""
 
-        msg = 'SETTINGS MANAGER: Setting profile to: %s' % profile
+        msg = f'SETTINGS MANAGER: Setting profile to: {profile}'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         oldVoiceLocale = self.getVoiceLocale('default')
@@ -432,7 +436,7 @@ class SettingsManager(object):
             orca_i18n.setLocaleForMessages(newVoiceLocale)
             orca_i18n.setLocaleForGUI(newVoiceLocale)
 
-        msg = 'SETTINGS MANAGER: Profile set to: %s' % profile
+        msg = f'SETTINGS MANAGER: Profile set to: {profile}'
         debug.println(debug.LEVEL_INFO, msg, True)
 
     def removeProfile(self, profile):
@@ -551,12 +555,12 @@ class SettingsManager(object):
     def saveSettings(self, script, general, pronunciations, keybindings):
         """Save the settings provided for the script provided."""
 
-        msg = 'SETTINGS MANAGER: Saving settings for %s (app: %s)' % (script, script.app)
+        msg = f'SETTINGS MANAGER: Saving settings for {script} (app: {script.app})'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         app = script.app
         if app:
-            self._saveAppSettings(app.name, general, pronunciations, keybindings)
+            self._saveAppSettings(AXObject.get_name(app), general, pronunciations, keybindings)
             return
 
         # Assign current profile
@@ -573,7 +577,7 @@ class SettingsManager(object):
         self._setProfilePronunciations(pronunciations)
         self._setProfileKeybindings(keybindings)
 
-        msg = 'SETTINGS MANAGER: Saving for backend %s' % self._backend
+        msg = f'SETTINGS MANAGER: Saving for backend {self._backend}'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         self._backend.saveProfileSettings(self.profile,
@@ -581,7 +585,7 @@ class SettingsManager(object):
                                           self.profilePronunciations,
                                           self.profileKeybindings)
 
-        msg = 'SETTINGS MANAGER: Settings for %s (app: %s) saved' % (script, script.app)
+        msg = f'SETTINGS MANAGER: Settings for {script} (app: {script.app}) saved'
         debug.println(debug.LEVEL_INFO, msg, True)
 
         return self._enableAccessibility()
@@ -632,7 +636,7 @@ class SettingsManager(object):
         if not app:
             return None
 
-        appPrefs = self._backend.getAppSettings(app.name)
+        appPrefs = self._backend.getAppSettings(AXObject.get_name(app))
         profiles = appPrefs.get('profiles', {})
         profilePrefs = profiles.get(self.profile, {})
         general = profilePrefs.get('general', {})
@@ -656,14 +660,14 @@ class SettingsManager(object):
         for key in self._appPronunciations.keys():
             self.pronunciations.pop(key)
 
-        prefs = self._backend.getAppSettings(script.app.name)
+        prefs = self._backend.getAppSettings(AXObject.get_name(script.app))
         profiles = prefs.get('profiles', {})
         profilePrefs = profiles.get(self.profile, {})
 
         self._appGeneral = profilePrefs.get('general', {})
         self._appKeybindings = profilePrefs.get('keybindings', {})
         self._appPronunciations = profilePrefs.get('pronunciations', {})
-        self._activeApp = script.app.name
+        self._activeApp = AXObject.get_name(script.app)
 
         self._loadProfileSettings()
         self._mergeSettings()
